from unstract.sdk.adapters.llm.no_op.src.no_op_llm import NoOpLLM

metadata = {
    "name": NoOpLLM.__name__,
    "version": "1.0.0",
    "adapter": NoOpLLM,
    "description": "NoOp LLM adapter",
    "is_active": True,
}

__all__ = ["NoOpLLM"]



from typing import Any

from llama_index.core import MockEmbedding


class NoOpCustomEmbedding(MockEmbedding):

    embed_dim: int

    def __init__(self, embed_dim: int, wait_time: float, **kwargs: Any) -> None:
        """Init params."""
        super().__init__(embed_dim=embed_dim, **kwargs, wait_time=wait_time)



from typing import Any, Union

from llama_index.core.callbacks.schema import EventPayload
from llama_index.core.llms import ChatResponse, CompletionResponse


class Constants:
    DEFAULT_TOKEN_COUNT = 0


class TokenCounter:
    prompt_llm_token_count: int
    completion_llm_token_count: int
    total_llm_token_count: int = 0
    total_embedding_token_count: int = 0

    def __init__(self, input_tokens, output_tokens):
        self.prompt_llm_token_count = input_tokens
        self.completion_llm_token_count = output_tokens
        self.total_llm_token_count = (
            self.prompt_llm_token_count + self.completion_llm_token_count
        )

    # TODO: Add unit test cases for the following function
    #  for ease of manintenance
    @staticmethod
    def get_llm_token_counts(payload: dict[str, Any]):
        prompt_tokens = Constants.DEFAULT_TOKEN_COUNT
        completion_tokens = Constants.DEFAULT_TOKEN_COUNT
        if EventPayload.PROMPT in payload:
            response = payload.get(EventPayload.COMPLETION)
            (
                prompt_tokens,
                completion_tokens,
            ) = TokenCounter._get_tokens_from_response(response)
        elif EventPayload.MESSAGES in payload:
            response = payload.get(EventPayload.RESPONSE)
            if response:
                (
                    prompt_tokens,
                    completion_tokens,
                ) = TokenCounter._get_tokens_from_response(response)

        token_counter = TokenCounter(
            input_tokens=prompt_tokens,
            output_tokens=completion_tokens,
        )
        return token_counter

    @staticmethod
    def _get_tokens_from_response(
        response: Union[CompletionResponse, ChatResponse, dict]
    ) -> tuple[int, int]:
        """Get the token counts from a raw response."""
        prompt_tokens, completion_tokens = 0, 0
        if isinstance(response, CompletionResponse) or isinstance(
            response, ChatResponse
        ):
            raw_response = response.raw
            if not isinstance(raw_response, dict):
                raw_response = dict(raw_response)

            usage = raw_response.get("usage", None)
        if usage is None:
            if (
                hasattr(response, "additional_kwargs")
                and "prompt_tokens" in response.additional_kwargs
            ):
                usage = response.additional_kwargs
            elif hasattr(response, "raw"):
                completion_raw = response.raw
                if ("_raw_response" in completion_raw) and hasattr(
                    completion_raw["_raw_response"], "usage_metadata"
                ):
                    usage = completion_raw["_raw_response"].usage_metadata
                    prompt_tokens = usage.prompt_token_count
                    completion_tokens = usage.candidates_token_count
                    return prompt_tokens, completion_tokens
                elif "inputTextTokenCount" in completion_raw:
                    prompt_tokens = completion_raw["inputTextTokenCount"]
                    if "results" in completion_raw:
                        result_list: list = completion_raw["results"]
                        if len(result_list) > 0:
                            result: dict = result_list[0]
                            if "tokenCount" in result:
                                completion_tokens = result.get("tokenCount", 0)
                    return prompt_tokens, completion_tokens
                else:
                    usage = response.raw
            else:
                usage = response

        if not isinstance(usage, dict):
            usage = usage.model_dump()

        possible_input_keys = (
            "prompt_tokens",
            "input_tokens",
            "prompt_eval_count",
        )
        possible_output_keys = (
            "completion_tokens",
            "output_tokens",
            "eval_count",
        )

        prompt_tokens = 0
        for input_key in possible_input_keys:
            if input_key in usage:
                prompt_tokens = int(usage[input_key])
                break

        completion_tokens = 0
        for output_key in possible_output_keys:
            if output_key in usage:
                completion_tokens = int(usage[output_key])
                break

        return prompt_tokens, completion_tokens



import logging
import os
from typing import Any, Optional

from llama_index.core.vector_stores.types import BasePydanticVectorStore
from llama_index.vector_stores.qdrant import QdrantVectorStore
from qdrant_client import QdrantClient
from qdrant_client.http.exceptions import UnexpectedResponse

from unstract.sdk.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.adapters.vectordb.helper import VectorDBHelper
from unstract.sdk.adapters.vectordb.vectordb_adapter import VectorDBAdapter
from unstract.sdk.exceptions import VectorDBError

logger = logging.getLogger(__name__)


class Constants:
    URL = "url"
    API_KEY = "api_key"


class Qdrant(VectorDBAdapter):
    def __init__(self, settings: dict[str, Any]):
        self._config = settings
        self._client: Optional[QdrantClient] = None
        self._collection_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        self._vector_db_instance = self._get_vector_db_instance()
        super().__init__("Qdrant", self._vector_db_instance)

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "qdrant|41f64fda-2e4c-4365-89fd-9ce91bee74d0"

    @staticmethod
    def get_name() -> str:
        return "Qdrant"

    @staticmethod
    def get_description() -> str:
        return "Qdrant LLM"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/qdrant.png"

     

    def get_vector_db_instance(self) -> BasePydanticVectorStore:
        return self._vector_db_instance

    def _get_vector_db_instance(self) -> BasePydanticVectorStore:
        try:
            self._collection_name = VectorDBHelper.get_collection_name(
                self._config.get(VectorDbConstants.VECTOR_DB_NAME),
                self._config.get(VectorDbConstants.EMBEDDING_DIMENSION),
            )
            url = self._config.get(Constants.URL)
            api_key: Optional[str] = self._config.get(Constants.API_KEY, None)
            if api_key:
                self._client = QdrantClient(url=url, api_key=api_key)
            else:
                self._client = QdrantClient(url=url)
            vector_db: BasePydanticVectorStore = QdrantVectorStore(
                collection_name=self._collection_name,
                client=self._client,
                url=url,
                api_key=api_key,
            )
            return vector_db
        except Exception as e:
            raise self.parse_vector_db_err(e) from e

    def test_connection(self) -> bool:
        try:
            vector_db = self.get_vector_db_instance()
            test_result: bool = VectorDBHelper.test_vector_db_instance(
                vector_store=vector_db
            )
            # Delete the collection that was created for testing
            if self._client is not None:
                self._client.delete_collection(self._collection_name)
            return test_result
        except Exception as e:
            raise self.parse_vector_db_err(e) from e

    def close(self, **kwargs: Any) -> None:
        if self._client:
            self._client.close(**kwargs)

    @staticmethod
    def parse_vector_db_err(e: Exception) -> VectorDBError:
        # Avoid wrapping VectorDBError objects again
        if isinstance(e, VectorDBError):
            return e

        if isinstance(e, UnexpectedResponse):
            msg = str(e)
            if e.reason_phrase == "Not Found":
                msg = "Unable to connect to Qdrant, please check vector DB settings."
            elif e.reason_phrase == "Forbidden":
                msg = "Unable to access Qdrant, please check the API key provided."
            return VectorDBError(message=msg, actual_err=e)
        else:
            status_code = None
            if "client has been closed" in str(e):
                status_code = 503
            elif "timeout" in str(e):
                status_code = 504
            return VectorDBError(message=str(e), actual_err=e, status_code=status_code)



import os
from typing import Any

from llama_index.core.llms import LLM
from llama_index.llms.azure_openai import AzureOpenAI

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.llm.constants import LLMKeys
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter


class Constants:
    MODEL = "model"
    DEPLOYMENT_NAME = "deployment_name"
    API_KEY = "api_key"
    API_VERSION = "api_version"
    MAX_RETRIES = "max_retries"
    MAX_TOKENS = "max_tokens"
    AZURE_ENDPONT = "azure_endpoint"
    API_TYPE = "azure"
    TIMEOUT = "timeout"
    DEFAULT_MODEL = "gpt-35-turbo"


class AzureOpenAILLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("AzureOpenAI")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "azureopenai|592d84b9-fe03-4102-a17e-6b391f32850b"

    @staticmethod
    def get_name() -> str:
        return "AzureOpenAI"

    @staticmethod
    def get_description() -> str:
        return "AzureOpenAI LLM"

    @staticmethod
    def get_provider() -> str:
        return "azure"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/AzureopenAI.png"

    def get_llm_instance(self) -> LLM:
        max_retries = int(
            self.config.get(Constants.MAX_RETRIES, LLMKeys.DEFAULT_MAX_RETRIES)
        )
        max_tokens = self.config.get(Constants.MAX_TOKENS)
        max_tokens = int(max_tokens) if max_tokens else None
        try:
            llm: LLM = AzureOpenAI(
                model=self.config.get(Constants.MODEL, Constants.DEFAULT_MODEL),
                deployment_name=str(self.config.get(Constants.DEPLOYMENT_NAME)),
                api_key=str(self.config.get(Constants.API_KEY)),
                api_version=str(self.config.get(Constants.API_VERSION)),
                azure_endpoint=str(self.config.get(Constants.AZURE_ENDPONT)),
                api_type=Constants.API_TYPE,
                temperature=0,
                timeout=float(
                    self.config.get(Constants.TIMEOUT, LLMKeys.DEFAULT_TIMEOUT)
                ),
                max_retries=max_retries,
                max_tokens=max_tokens,
            )
            return llm
        except Exception as e:
            raise AdapterError(str(e))



from typing import Optional


def resolve_err_status_code(client_status_code: int) -> int:
    """Resolves the status code to return in case of errors.

    Returns a status code which should be returned from Unstract based on
    the status code from the client.

    - 502 is returned for 500 client errors
    - 4xx is returned for 4xx client errors

    Args:
        client_status_code (int): Status code from client library

    Returns:
        int: Status code that Unstract should return
    """
    if client_status_code == 500:
        return 502

    # 4xx status codes returned as is
    return client_status_code


class SdkError(Exception):
    DEFAULT_MESSAGE = "Something went wrong"
    actual_err: Optional[Exception] = None
    status_code: Optional[int] = None

    def __init__(
        self,
        message: str = DEFAULT_MESSAGE,
        status_code: Optional[int] = None,
        actual_err: Optional[Exception] = None,
    ):
        super().__init__(message)
        # Make it user friendly wherever possible
        self.message = message
        if actual_err:
            self.actual_err = actual_err

        # Setting status code for error
        if status_code:
            self.status_code = status_code
        elif actual_err:
            if hasattr(actual_err, "status_code"):  # Most providers
                self.status_code = resolve_err_status_code(actual_err.status_code)
            elif hasattr(actual_err, "http_status"):  # Few providers like Mistral
                self.status_code = resolve_err_status_code(actual_err.http_status)

    def __str__(self) -> str:
        return self.message


class IndexingError(SdkError):
    def __init__(self, message: str = "", **kwargs):
        if "404" in message:
            message = "Index not found. Please check vector DB settings."
        super().__init__(message, **kwargs)


class LLMError(SdkError):
    DEFAULT_MESSAGE = "Error ocurred related to LLM"


class EmbeddingError(SdkError):
    DEFAULT_MESSAGE = "Error ocurred related to embedding"


class VectorDBError(SdkError):
    DEFAULT_MESSAGE = "Error ocurred related to vector DB"


class X2TextError(SdkError):
    DEFAULT_MESSAGE = "Error ocurred related to text extractor"


class OCRError(SdkError):
    DEFAULT_MESSAGE = "Error ocurred related to OCR"


class RateLimitError(SdkError):
    DEFAULT_MESSAGE = "Running into rate limit errors, please try again later"


class FileStorageError(SdkError):
    DEFAULT_MESSAGE = (
        "Error while connecting with the storage. "
        "Please check the configuration credentials"
    )


class FileOperationError(SdkError):
    DEFAULT_MESSAGE = (
        "Error while performing operation on the file. "
        "Please check specific storage error for "
        "further information"
    )



from abc import ABC
from typing import Any, Optional

from unstract.sdk.adapters.base import Adapter
from unstract.sdk.adapters.enums import AdapterTypes
from unstract.sdk.adapters.x2text.dto import TextExtractionResult
from unstract.sdk.file_storage import FileStorage, FileStorageProvider


class X2TextAdapter(Adapter, ABC):
    def __init__(self, name: str):
        super().__init__(name)
        self.name = name

    @staticmethod
    def get_id() -> str:
        return ""

    @staticmethod
    def get_name() -> str:
        return ""

    @staticmethod
    def get_description() -> str:
        return ""

    @staticmethod
    def get_icon() -> str:
        return ""

    @staticmethod
    def get_adapter_type() -> AdapterTypes:
        return AdapterTypes.X2TEXT

    def test_connection(self) -> bool:
        return False

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        **kwargs: dict[Any, Any],
    ) -> TextExtractionResult:
        return TextExtractionResult(
            extracted_text="extracted text", extraction_metadata=None
        )



import os
import time
from typing import Any

from llama_index.core.embeddings import BaseEmbedding

from unstract.sdk.adapters.embedding.embedding_adapter import EmbeddingAdapter
from unstract.sdk.adapters.embedding.no_op.src.no_op_custom_embedding import (
    NoOpCustomEmbedding,
)


class NoOpEmbedding(EmbeddingAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("NoOpCustomEmbedding")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "noOpEmbedding|ff223003-fee8-4079-b288-e86215e6b39a"

    @staticmethod
    def get_name() -> str:
        return "No Op Embedding"

    @staticmethod
    def get_description() -> str:
        return "No Op Embedding"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/noOpEmbedding.png"

    @staticmethod
    def get_provider() -> str:
        return "NoOp"

    def get_embedding_instance(self) -> BaseEmbedding:
        embedding: BaseEmbedding = NoOpCustomEmbedding(
            embed_dim=1, wait_time=self.config.get("wait_time")
        )
        return embedding

    def test_connection(self) -> bool:
        time.sleep(self.config.get("wait_time"))
        return True



import logging
from typing import Any

import fsspec
from fsspec import AbstractFileSystem

from unstract.sdk.exceptions import FileOperationError, FileStorageError
from unstract.sdk.file_storage.provider import FileStorageProvider

logger = logging.getLogger(__name__)


class FileStorageHelper:
    @staticmethod
    def file_storage_init(
        provider: FileStorageProvider, **storage_config: dict[str, Any]
    ) -> AbstractFileSystem:
        """Initialises file storage based on provider.

        Args:
            provider (FileStorageProvider): Provider
            storage_config : Storage config params based on the provider.
            Sent as-is to the provider implementation.

        Returns:
            NA
        """

        try:
            protocol = provider.value
            if provider == FileStorageProvider.LOCAL:
                # Hard set auto_mkdir to True as default
                storage_config.update({"auto_mkdir": True})  # type: ignore
            elif provider in [FileStorageProvider.MINIO]:
                # Initialise using s3 for Minio
                protocol = FileStorageProvider.S3.value

            fs = fsspec.filesystem(
                protocol=protocol,
                **storage_config,
            )
            logger.debug(f"Connected to {provider.value} file system")
        except KeyError as e:
            logger.error(
                f"Error in initialising {provider.value} "
                f"file system because of missing config {e}"
            )
            raise FileStorageError(str(e)) from e
        except Exception as e:
            logger.error(f"Error in initialising {provider.value} " f"file system {e}")
            raise FileStorageError(str(e)) from e
        return fs

    @staticmethod
    def local_file_system_init() -> AbstractFileSystem:
        """Initialises FileStorage backed up by Local file system.

        Returns:
            NA
        """
        try:
            fs = fsspec.filesystem(protocol=FileStorageProvider.LOCAL.value)
            logger.debug(f"Connected to {FileStorageProvider.LOCAL.value} file system")
            return fs
        except Exception as e:
            logger.error(
                f"Error in initialising {FileStorageProvider.GCS.value}"
                f" file system {e}"
            )
            raise FileStorageError(str(e)) from e


def skip_local_cache(func):
    """Helper function/decorator for handling FileNotFound exception and making
    sure that the error is not because of stale cache.

    Args:
        func: The original function that is called in the context

    Returns:
        NA
    """

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except FileNotFoundError:
            _handle_file_not_found(func, *args, **kwargs)
        except Exception as e:
            raise FileOperationError(str(e)) from e

    return wrapper


def _handle_file_not_found(func, *args, **kwargs):
    """Helper function for handling FileNotFound exception and making sure that
    the error is not because of stale cache.

    Args:
        func: The original function that is called in the context
        args: The context of the function call as an array
        kwargs: args to the function being called in this context

    Returns:
        NA
    """
    try:
        # FileNotFound could have been caused by stale cache.
        # Hence invalidate cache and retry again
        args[0].fs.invalidate_cache()
        return func(*args, **kwargs)
    except Exception as e:
        if isinstance(e, FileNotFoundError):
            raise e
        else:
            raise FileOperationError(str(e)) from e



import json
import logging
from typing import Any, Callable, Optional

from deprecated import deprecated
from llama_index.core import Document
from llama_index.core.node_parser import SentenceSplitter
from llama_index.core.vector_stores import (
    FilterOperator,
    MetadataFilter,
    MetadataFilters,
    VectorStoreQuery,
    VectorStoreQueryResult,
)

from unstract.sdk.adapter import ToolAdapter
from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.vectordb.no_op.src.no_op_custom_vectordb import (
    NoOpCustomVectorDB,
)
from unstract.sdk.adapters.x2text.constants import X2TextConstants
from unstract.sdk.adapters.x2text.dto import TextExtractionResult
from unstract.sdk.adapters.x2text.llm_whisperer.src import LLMWhisperer
from unstract.sdk.constants import LogLevel
from unstract.sdk.embedding import Embedding
from unstract.sdk.exceptions import IndexingError, SdkError, VectorDBError, X2TextError
from unstract.sdk.file_storage import FileStorage, FileStorageProvider
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.utils import ToolUtils
from unstract.sdk.utils.common_utils import capture_metrics, log_elapsed
from unstract.sdk.vector_db import VectorDB
from unstract.sdk.x2txt import X2Text

logger = logging.getLogger(__name__)


class Constants:
    TOP_K = 5


class Index:
    def __init__(
        self,
        tool: BaseTool,
        run_id: Optional[str] = None,
        capture_metrics: bool = False,
    ):
        # TODO: Inherit from StreamMixin and avoid using BaseTool
        self.tool = tool
        self._run_id = run_id
        self._capture_metrics = capture_metrics
        self._metrics = {}

    @capture_metrics
    def query_index(
        self,
        embedding_instance_id: str,
        vector_db_instance_id: str,
        doc_id: str,
        usage_kwargs: dict[Any, Any] = {},
    ):
        embedding = Embedding(
            tool=self.tool,
            adapter_instance_id=embedding_instance_id,
            usage_kwargs=usage_kwargs,
        )

        vector_db = VectorDB(
            tool=self.tool,
            adapter_instance_id=vector_db_instance_id,
            embedding=embedding,
        )

        try:
            self.tool.stream_log(
                f">>> Querying '{vector_db_instance_id}' for {doc_id}..."
            )
            try:
                doc_id_eq_filter = MetadataFilter.from_dict(
                    {
                        "key": "doc_id",
                        "operator": FilterOperator.EQ,
                        "value": doc_id,
                    }
                )
                filters = MetadataFilters(filters=[doc_id_eq_filter])
                q = VectorStoreQuery(
                    query_embedding=embedding.get_query_embedding(" "),
                    doc_ids=[doc_id],
                    filters=filters,
                    similarity_top_k=Constants.TOP_K,
                )
            except Exception as e:
                self.tool.stream_log(
                    f"Error while building vector DB query: {e}", level=LogLevel.ERROR
                )
                raise VectorDBError(
                    f"Failed to construct query for {vector_db}: {e}", actual_err=e
                )
            try:
                n: VectorStoreQueryResult = vector_db.query(query=q)
                if len(n.nodes) > 0:
                    self.tool.stream_log(f"Found {len(n.nodes)} nodes for {doc_id}")
                    all_text = ""
                    for node in n.nodes:
                        all_text += node.get_content()
                    return all_text
                else:
                    self.tool.stream_log(f"No nodes found for {doc_id}")
                    return None
            except Exception as e:
                self.tool.stream_log(
                    f"Error while executing vector DB query: {e}", level=LogLevel.ERROR
                )
                raise VectorDBError(
                    f"Failed to execute query on {vector_db}: {e}", actual_err=e
                )
        finally:
            vector_db.close()

    @log_elapsed(operation="EXTRACTION")
    def extract_text(
        self,
        x2text_instance_id: str,
        file_path: str,
        output_file_path: Optional[str] = None,
        enable_highlight: bool = False,
        usage_kwargs: dict[Any, Any] = {},
        process_text: Optional[Callable[[str], str]] = None,
        fs: FileStorage = FileStorage(FileStorageProvider.LOCAL),
        tags: Optional[list[str]] = None,
    ) -> str:
        """Extracts text from a document.

        Uses the configured service to perform the extraction
        - LLMWhisperer
        - Unstructured IO Community / Enterprise
        - Llama Parse

        Args:
            x2text_instance_id (str): UUID of the text extractor
            file_path (str): Path to the file
            output_file_path (Optional[str], optional): File path to write
                the extracted contents into. Defaults to None.
            enable_highlight (bool, optional): Flag to provide highlighting metadata.
                Defaults to False.
            usage_kwargs (dict[Any, Any], optional): Dict to capture usage.
                Defaults to {}.
            process_text (Optional[Callable[[str], str]], optional): Optional function
                to post-process the text. Defaults to None.
            tags: (Optional[list[str]], optional): Tags

        Raises:
            IndexingError: Errors during text extraction
        """
        self.tool.stream_log("Extracting text from input file")
        extracted_text = ""
        x2text = X2Text(
            tool=self.tool,
            adapter_instance_id=x2text_instance_id,
            usage_kwargs=usage_kwargs,
        )
        try:
            if enable_highlight and isinstance(x2text.x2text_instance, LLMWhisperer):
                process_response: TextExtractionResult = x2text.process(
                    input_file_path=file_path,
                    output_file_path=output_file_path,
                    enable_highlight=enable_highlight,
                    tags=tags,
                    fs=fs,
                )
                whisper_hash_value = process_response.extraction_metadata.whisper_hash
                metadata = {X2TextConstants.WHISPER_HASH: whisper_hash_value}
                self.tool.update_exec_metadata(metadata)
            else:
                process_response: TextExtractionResult = x2text.process(
                    input_file_path=file_path,
                    output_file_path=output_file_path,
                    tags=tags,
                    fs=fs,
                )
            extracted_text = process_response.extracted_text
        # TODO: Handle prepend of context where error is raised and remove this
        except AdapterError as e:
            msg = f"Error from text extractor '{x2text.x2text_instance.get_name()}'. "
            msg += str(e)
            raise X2TextError(msg) from e
        if process_text:
            try:
                result = process_text(extracted_text)
                if isinstance(result, str):
                    extracted_text = result
                else:
                    logger.warning("'process_text' is expected to return an 'str'")
            except Exception as e:
                logger.error(
                    f"Error occured inside callable 'process_text': {e}\n"
                    "continuing processing..."
                )
        return extracted_text

    # TODO: Reduce the number of params by some dataclass
    @log_elapsed(operation="CHECK_AND_INDEX(overall)")
    @capture_metrics
    def index(
        self,
        tool_id: str,
        embedding_instance_id: str,
        vector_db_instance_id: str,
        x2text_instance_id: str,
        file_path: str,
        chunk_size: int,
        chunk_overlap: int,
        reindex: bool = False,
        file_hash: Optional[str] = None,
        output_file_path: Optional[str] = None,
        enable_highlight: bool = False,
        usage_kwargs: dict[Any, Any] = {},
        process_text: Optional[Callable[[str], str]] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        tags: Optional[list[str]] = None,
    ) -> str:
        """Indexes an individual file using the passed arguments.

        Args:
            tool_id (str): UUID of the tool (workflow_id in case it's called
                from workflow)
            embedding_instance_id (str): UUID of the embedding service configured
            vector_db_instance_id (str): UUID of the vector DB configured
            x2text_instance_id (str): UUID of the x2text adapter configured.
                This is to extract text from documents.
            file_path (str): Path to the file that needs to be indexed.
            chunk_size (int): Chunk size to be used for indexing
            chunk_overlap (int): Overlap in chunks to be used for indexing
            reindex (bool, optional): Flag to denote if document should be
                re-indexed if its already indexed. Defaults to False.
            file_hash (Optional[str], optional): SHA256 hash of the file.
                Defaults to None. If None, the hash is generated.
            output_file_path (Optional[str], optional): File path to write
                the extracted contents into. Defaults to None.
            fs (FileStorage): file storage object to perfrom file operations
            tags (Optional[list[str]], optional): List of tags to be associated with
                the indexed document.

        Returns:
            str: A unique ID for the file and indexing arguments combination
        """
        doc_id = self.generate_index_key(
            vector_db=vector_db_instance_id,
            embedding=embedding_instance_id,
            x2text=x2text_instance_id,
            chunk_size=str(chunk_size),
            chunk_overlap=str(chunk_overlap),
            file_path=file_path,
            file_hash=file_hash,
            fs=fs,
        )
        self.tool.stream_log(f"Checking if doc_id {doc_id} exists")

        embedding = Embedding(
            tool=self.tool,
            adapter_instance_id=embedding_instance_id,
            usage_kwargs=usage_kwargs,
        )

        vector_db = VectorDB(
            tool=self.tool,
            adapter_instance_id=vector_db_instance_id,
            embedding=embedding,
        )

        try:
            # Checking if document is already indexed against doc_id
            doc_id_eq_filter = MetadataFilter.from_dict(
                {"key": "doc_id", "operator": FilterOperator.EQ, "value": doc_id}
            )
            filters = MetadataFilters(filters=[doc_id_eq_filter])
            q = VectorStoreQuery(
                query_embedding=embedding.get_query_embedding(" "),
                doc_ids=[doc_id],
                filters=filters,
            )

            doc_id_found = False
            try:
                n: VectorStoreQueryResult = vector_db.query(query=q)
                if len(n.nodes) > 0:
                    doc_id_found = True
                    self.tool.stream_log(f"Found {len(n.nodes)} nodes for {doc_id}")
                else:
                    self.tool.stream_log(f"No nodes found for {doc_id}")
            except Exception as e:
                self.tool.stream_log(
                    f"Error querying {vector_db_instance_id}: {e}, proceeding to index",
                    level=LogLevel.ERROR,
                )

            if doc_id_found and not reindex:
                self.tool.stream_log(f"File was indexed already under {doc_id}")
                if output_file_path and not fs.exists(output_file_path):
                    # Added this as a workaround to handle extraction
                    # for documents uploaded twice in different projects.
                    # to be reconsidered after permanent fixes.
                    extracted_text = self.extract_text(
                        x2text_instance_id=x2text_instance_id,
                        file_path=file_path,
                        output_file_path=output_file_path,
                        enable_highlight=enable_highlight,
                        usage_kwargs=usage_kwargs,
                        process_text=process_text,
                        fs=fs,
                        tags=tags,
                    )
                return doc_id

            extracted_text = self.extract_text(
                x2text_instance_id=x2text_instance_id,
                file_path=file_path,
                output_file_path=output_file_path,
                enable_highlight=enable_highlight,
                usage_kwargs=usage_kwargs,
                process_text=process_text,
                tags=tags,
                fs=fs,
            )
            if not extracted_text:
                raise IndexingError("No text available to index")

            # For No-op adapters, addition of nodes to vectorDB should not happen
            # and this has to be handled in the adapter level. But there are a
            # few challenges considering callback manager and upstream Llama index
            # method invocations. Hence, making this check here and returning
            # the doc id to maintain the legacy flow of adapters.

            if isinstance(
                vector_db.get_vector_db(
                    adapter_instance_id=vector_db_instance_id, embedding_dimension=1
                ),
                (NoOpCustomVectorDB),
            ):
                return doc_id

            self.index_to_vector_db(
                vector_db=vector_db,
                embedding=embedding,
                chunk_size=chunk_size,
                chunk_overlap=chunk_overlap,
                doc_id=doc_id,
                text_to_idx=extracted_text,
                doc_id_found=doc_id_found,
            )
            return doc_id
        finally:
            vector_db.close()

    @log_elapsed(operation="INDEXING")
    def index_to_vector_db(
        self,
        vector_db: VectorDB,
        embedding: Embedding,
        chunk_size: int,
        chunk_overlap: int,
        text_to_idx: str,
        doc_id: str,
        doc_id_found: bool,
    ):
        self.tool.stream_log("Indexing file...")
        full_text = [
            {
                "section": "full",
                "text_contents": text_to_idx,
            }
        ]
        # Check if chunking is required
        documents = []
        for item in full_text:
            text = item["text_contents"]
            document = Document(
                text=text,
                doc_id=doc_id,
                metadata={"section": item["section"]},
            )
            document.id_ = doc_id
            documents.append(document)
        self.tool.stream_log(f"Number of documents: {len(documents)}")

        if doc_id_found:
            # Delete the nodes for the doc_id
            try:
                vector_db.delete(ref_doc_id=doc_id)
                self.tool.stream_log(f"Deleted nodes for {doc_id}")
            except Exception as e:
                self.tool.stream_log(
                    f"Error deleting nodes for {doc_id}: {e}",
                    level=LogLevel.ERROR,
                )
                raise SdkError(f"Error deleting nodes for {doc_id}: {e}") from e

        try:
            if chunk_size == 0:
                parser = SentenceSplitter.from_defaults(
                    chunk_size=len(documents[0].text) + 10,
                    chunk_overlap=0,
                    callback_manager=embedding.get_callback_manager(),
                )
                nodes = parser.get_nodes_from_documents(documents, show_progress=True)
                node = nodes[0]
                node.embedding = embedding.get_query_embedding(" ")
                vector_db.add(doc_id, nodes=[node])
                self.tool.stream_log("Added node to vector db")
            else:
                self.tool.stream_log("Adding nodes to vector db...")
                # TODO: Phase 2:
                # Post insertion to VDB, use query using doc_id and
                # store all the VDB ids to a table against the doc_id
                # During deletion for cases where metadata filtering
                # does not work, these ids can be used for direct deletion
                # This new table will also act like an audit trail for
                # all nodes that were added to the VDB by Unstract
                # Once this is in place, the overridden implementation
                # of prefixing ids with doc_id before adding to VDB
                # can be removed
                vector_db.index_document(
                    documents,
                    chunk_size=chunk_size,
                    chunk_overlap=chunk_overlap,
                    show_progress=True,
                )
        except Exception as e:
            self.tool.stream_log(
                f"Error adding nodes to vector db: {e}",
                level=LogLevel.ERROR,
            )
            raise IndexingError(str(e)) from e

        self.tool.stream_log("File has been indexed successfully")
        return

    def generate_index_key(
        self,
        vector_db: str,
        embedding: str,
        x2text: str,
        chunk_size: str,
        chunk_overlap: str,
        file_path: Optional[str] = None,
        file_hash: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        """Generates a unique ID useful for identifying files during indexing.

        Args:
            vector_db (str): UUID of the vector DB adapter
            embedding (str): UUID of the embedding adapter
            x2text (str): UUID of the X2Text adapter
            chunk_size (str): Chunk size for indexing
            chunk_overlap (str): Chunk overlap for indexing
            file_path (Optional[str]): Path to the file that needs to be indexed.
                Defaults to None. One of file_path or file_hash needs to be specified.
            file_hash (Optional[str], optional): SHA256 hash of the file.
                Defaults to None. If None, the hash is generated with file_path.
            fs (FileStorage): file storage object to perfrom file operations

        Returns:
            str: Key representing unique ID for a file
        """
        if not file_path and not file_hash:
            raise ValueError("One of `file_path` or `file_hash` need to be provided")

        if not file_hash:
            file_hash = fs.get_hash_from_file(path=file_path)

        # Whole adapter config is used currently even though it contains some keys
        # which might not be relevant to indexing. This is easier for now than
        # marking certain keys of the adapter config as necessary.
        index_key = {
            "file_hash": file_hash,
            "vector_db_config": ToolAdapter.get_adapter_config(self.tool, vector_db),
            "embedding_config": ToolAdapter.get_adapter_config(self.tool, embedding),
            "x2text_config": ToolAdapter.get_adapter_config(self.tool, x2text),
            # Typed and hashed as strings since the final hash is persisted
            # and this is required to be backward compatible
            "chunk_size": str(chunk_size),
            "chunk_overlap": str(chunk_overlap),
        }
        # JSON keys are sorted to ensure that the same key gets hashed even in
        # case where the fields are reordered.
        hashed_index_key = ToolUtils.hash_str(json.dumps(index_key, sort_keys=True))
        return hashed_index_key

    def get_metrics(self):
        return self._metrics

    def clear_metrics(self):
        self._metrics = {}

    @deprecated(version="0.45.0", reason="Use generate_index_key() instead")
    def generate_file_id(
        self,
        tool_id: str,
        vector_db: str,
        embedding: str,
        x2text: str,
        chunk_size: str,
        chunk_overlap: str,
        file_path: Optional[str] = None,
        file_hash: Optional[str] = None,
    ) -> str:
        return self.generate_index_key(
            vector_db,
            embedding,
            x2text,
            chunk_size,
            chunk_overlap,
            file_path,
            file_hash,
            fs=FileStorage(provider=FileStorageProvider.LOCAL),
        )

    @deprecated(version="0.50.0", reason="Use index() instead")
    def index_file(
        self,
        tool_id: str,
        embedding_type: str,
        vector_db: str,
        x2text_adapter: str,
        file_path: str,
        chunk_size: int,
        chunk_overlap: int,
        reindex: bool = False,
        file_hash: Optional[str] = None,
        output_file_path: Optional[str] = None,
    ) -> str:
        return self.index(
            tool_id=tool_id,
            embedding_instance_id=embedding_type,
            vector_db_instance_id=vector_db,
            x2text_instance_id=x2text_adapter,
            file_path=file_path,
            chunk_size=chunk_size,
            chunk_overlap=chunk_overlap,
            reindex=reindex,
            file_hash=file_hash,
            output_file_path=output_file_path,
        )

    @deprecated("Deprecated class and method. Use Index and query_index() instead")
    def get_text_from_index(
        self, embedding_type: str, vector_db: str, doc_id: str
    ) -> Optional[str]:
        return self.query_index(
            embedding_instance_id=embedding_type,
            vector_db_instance_id=vector_db,
            doc_id=doc_id,
        )


# Legacy
ToolIndex = Index



import os
from enum import Enum


class ProcessingModes(Enum):
    OCR = "ocr"
    TEXT = "text"


class Modes(Enum):
    NATIVE_TEXT = "native_text"
    LOW_COST = "low_cost"
    HIGH_QUALITY = "high_quality"
    FORM = "form"


class OutputModes(Enum):
    LINE_PRINTER = "line-printer"
    DUMP_TEXT = "dump-text"
    TEXT = "text"


class HTTPMethod(Enum):
    GET = "GET"
    POST = "POST"


class WhispererHeader:
    UNSTRACT_KEY = "unstract-key"


class WhispererEndpoint:
    """Endpoints available at LLMWhisperer service."""

    TEST_CONNECTION = "test-connection"
    WHISPER = "whisper"
    STATUS = "whisper-status"
    RETRIEVE = "whisper-retrieve"


class WhispererEnv:
    """Env variables for LLMWhisperer.

    Can be used to alter behaviour at runtime.

    Attributes:
        POLL_INTERVAL: Time in seconds to wait before polling
            LLMWhisperer's status API. Defaults to 30s
        MAX_POLLS: Total number of times to poll the status API.
            Set to -1 to poll indefinitely. Defaults to -1
    """

    POLL_INTERVAL = "ADAPTER_LLMW_POLL_INTERVAL"
    MAX_POLLS = "ADAPTER_LLMW_MAX_POLLS"


class WhispererConfig:
    """Dictionary keys used to configure LLMWhisperer service."""

    URL = "url"
    PROCESSING_MODE = "processing_mode"
    MODE = "mode"
    OUTPUT_MODE = "output_mode"
    UNSTRACT_KEY = "unstract_key"
    MEDIAN_FILTER_SIZE = "median_filter_size"
    GAUSSIAN_BLUR_RADIUS = "gaussian_blur_radius"
    FORCE_TEXT_PROCESSING = "force_text_processing"
    LINE_SPLITTER_TOLERANCE = "line_splitter_tolerance"
    HORIZONTAL_STRETCH_FACTOR = "horizontal_stretch_factor"
    PAGES_TO_EXTRACT = "pages_to_extract"
    STORE_METADATA_FOR_HIGHLIGHTING = "store_metadata_for_highlighting"
    ADD_LINE_NOS = "add_line_nos"
    OUTPUT_JSON = "output_json"
    PAGE_SEPARATOR = "page_seperator"
    MARK_VERTICAL_LINES = "mark_vertical_lines"
    MARK_HORIZONTAL_LINES = "mark_horizontal_lines"


class WhisperStatus:
    """Values returned / used by /whisper-status endpoint."""

    PROCESSING = "processing"
    PROCESSED = "processed"
    DELIVERED = "delivered"
    UNKNOWN = "unknown"
    # Used for async processing
    WHISPER_HASH = "whisper-hash"
    STATUS = "status"


class WhispererDefaults:
    """Defaults meant for LLMWhisperer."""

    MEDIAN_FILTER_SIZE = 0
    GAUSSIAN_BLUR_RADIUS = 0.0
    FORCE_TEXT_PROCESSING = False
    LINE_SPLITTER_TOLERANCE = 0.75
    HORIZONTAL_STRETCH_FACTOR = 1.0
    POLL_INTERVAL = int(os.getenv(WhispererEnv.POLL_INTERVAL, 30))
    MAX_POLLS = int(os.getenv(WhispererEnv.MAX_POLLS, 30))
    PAGES_TO_EXTRACT = ""
    ADD_LINE_NOS = True
    OUTPUT_JSON = True
    PAGE_SEPARATOR = "<<< >>>"
    MARK_VERTICAL_LINES = False
    MARK_HORIZONTAL_LINES = False



import os
from typing import Any, Optional
from urllib.parse import quote_plus

from llama_index.core.vector_stores.types import VectorStore
from llama_index.vector_stores.supabase import SupabaseVectorStore
from vecs import Client

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.adapters.vectordb.helper import VectorDBHelper
from unstract.sdk.adapters.vectordb.vectordb_adapter import VectorDBAdapter


class Constants:
    DATABASE = "database"
    HOST = "host"
    PASSWORD = "password"
    PORT = "port"
    USER = "user"
    COLLECTION_NAME = "base_demo"


class Supabase(VectorDBAdapter):
    def __init__(self, settings: dict[str, Any]):
        self._config = settings
        self._client: Optional[Client] = None
        self._collection_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        self._vector_db_instance = self._get_vector_db_instance()
        super().__init__("Supabase", self._vector_db_instance)

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "supabase|e6998e3c-3595-48c0-a190-188dbd803858"

    @staticmethod
    def get_name() -> str:
        return "Supabase"

    @staticmethod
    def get_description() -> str:
        return "Supabase VectorDB"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/supabase.png"

     

    def get_vector_db_instance(self) -> VectorStore:
        return self._vector_db_instance

    def _get_vector_db_instance(self) -> VectorStore:
        try:
            dimension = self._config.get(
                VectorDbConstants.EMBEDDING_DIMENSION,
                VectorDbConstants.DEFAULT_EMBEDDING_SIZE,
            )
            self._collection_name = VectorDBHelper.get_collection_name(
                self._config.get(VectorDbConstants.VECTOR_DB_NAME),
                self._config.get(
                    VectorDbConstants.EMBEDDING_DIMENSION,
                    dimension,
                ),
            )
            user = str(self._config.get(Constants.USER))
            password = str(self._config.get(Constants.PASSWORD))
            encoded_password = quote_plus(str(password))
            host = str(self._config.get(Constants.HOST))
            port = str(self._config.get(Constants.PORT))
            db_name = str(self._config.get(Constants.DATABASE))

            postgres_connection_string = (
                f"postgresql://{user}:{encoded_password}@{host}:{port}/{db_name}"
            )
            vector_db: VectorStore = SupabaseVectorStore(
                postgres_connection_string=postgres_connection_string,
                collection_name=self._collection_name,
                dimension=dimension,
            )
            if vector_db is not None:
                self._client = vector_db.client
            return vector_db
        except Exception as e:
            raise AdapterError(str(e))

    def test_connection(self) -> bool:
        vector_db = self.get_vector_db_instance()
        test_result: bool = VectorDBHelper.test_vector_db_instance(
            vector_store=vector_db
        )
        # Delete the collection that was created for testing
        if self._client is not None:
            self._client.delete_collection(self._collection_name)
        return test_result

    def close(self, **kwargs: Any) -> None:
        if self._client:
            self._client.close()



from .replicate import ReplicateLLM

metadata = {
    "name": ReplicateLLM.__name__,
    "version": "1.0.0",
    "adapter": ReplicateLLM,
    "description": "Replicate LLM adapter",
    "is_active": True,
}

__all__ = ["ReplicateLLM"]



from typing import Any

from unstract.sdk.exceptions import FileStorageError
from unstract.sdk.file_storage import FileStorage, FileStorageProvider


class SharedTemporaryFileStorage(FileStorage):
    SUPPORTED_FILE_STORAGE_TYPES = [
        FileStorageProvider.MINIO.value,
        FileStorageProvider.REDIS.value,
    ]

    def __init__(
        self,
        provider: FileStorageProvider,
        **storage_config: dict[str, Any],
    ):
        if provider.value not in self.SUPPORTED_FILE_STORAGE_TYPES:
            raise FileStorageError(
                f"File storage provider is not supported in Shared Temporary mode. "
                f"Supported providers: {self.SUPPORTED_FILE_STORAGE_TYPES}"
            )
        if provider == FileStorageProvider.MINIO:
            super().__init__(provider, **storage_config)
        elif provider == FileStorageProvider.REDIS:
            super().__init__(provider)
        else:
            raise NotImplementedError



import logging
import os
import re
from typing import Any

from httpx import ConnectError, HTTPStatusError
from llama_index.core.llms import LLM
from llama_index.llms.ollama import Ollama

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.llm.constants import LLMKeys
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter

logger = logging.getLogger(__name__)


class Constants:
    MODEL = "model"
    API_KEY = "api_key"
    TIMEOUT = "timeout"
    BASE_URL = "base_url"
    JSON_MODE = "json_mode"
    CONTEXT_WINDOW = "context_window"
    MODEL_MISSING_ERROR = "try pulling it first"


class OllamaLLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("Ollama")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "ollama|4b8bd31a-ce42-48d4-9d69-f29c12e0f276"

    @staticmethod
    def get_name() -> str:
        return "Ollama"

    @staticmethod
    def get_description() -> str:
        return "Ollama AI LLM"

    @staticmethod
    def get_provider() -> str:
        return "ollama"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/ollama.png"

    def get_llm_instance(self) -> LLM:
        try:
            llm: LLM = Ollama(
                model=str(self.config.get(Constants.MODEL)),
                base_url=str(self.config.get(Constants.BASE_URL)),
                request_timeout=float(
                    self.config.get(Constants.TIMEOUT, LLMKeys.DEFAULT_TIMEOUT)
                ),
                json_mode=False,
                context_window=int(self.config.get(Constants.CONTEXT_WINDOW, 3900)),
                temperature=0.01,
            )
            return llm

        except ConnectError as connec_err:
            logger.error(f"Ollama server not running : {connec_err}")
            raise AdapterError(
                "Unable to connect to Ollama`s Server, "
                "please check if the server is up and running or"
                "if it is accepting connections."
            )
        except Exception as exc:
            logger.error(f"Error occured while getting llm instance:{exc}")
            raise AdapterError(str(exc))

    def test_connection(self) -> bool:
        try:
            llm = self.get_llm_instance()
            if not llm:
                return False
            response = llm.complete(
                "The capital of Tamilnadu is ",
                temperature=0.003,
            )
            response_lower_case: str = response.text.lower()
            find_match = re.search("chennai", response_lower_case)
            if find_match:
                return True
            else:
                return False
        except HTTPStatusError as http_err:
            if http_err.response:
                if (
                    http_err.response.status_code == 404
                    and Constants.MODEL_MISSING_ERROR in http_err.response.text
                ):
                    logger.error(
                        f"Error occured while sending requst to the model{http_err}"
                    )
                    raise AdapterError(
                        "Model under use is not found. Try pulling it first."
                    )
            raise AdapterError(
                f"Some issue while communicating with the model. "
                f"Details : {http_err.response.text}"
            )

        except Exception as e:
            logger.error(f"Error occured while testing adapter {e}")
            raise AdapterError(str(e))



from .open_ai import OpenAILLM

metadata = {
    "name": OpenAILLM.__name__,
    "version": "1.0.0",
    "adapter": OpenAILLM,
    "description": "OpenAI LLM adapter",
    "is_active": True,
}

__all__ = ["OpenAILLM"]



import json
import logging
import os
import time
from pathlib import Path
from typing import Any, Optional

import requests
from requests import Response
from requests.exceptions import ConnectionError, HTTPError, Timeout

from unstract.sdk.adapters.exceptions import ExtractorError
from unstract.sdk.adapters.utils import AdapterUtils
from unstract.sdk.adapters.x2text.constants import X2TextConstants
from unstract.sdk.adapters.x2text.dto import (
    TextExtractionMetadata,
    TextExtractionResult,
)
from unstract.sdk.adapters.x2text.llm_whisperer.src.constants import (
    HTTPMethod,
    OutputModes,
    ProcessingModes,
    WhispererConfig,
    WhispererDefaults,
    WhispererEndpoint,
    WhispererHeader,
    WhisperStatus,
)
from unstract.sdk.adapters.x2text.x2text_adapter import X2TextAdapter
from unstract.sdk.constants import MimeType
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class LLMWhisperer(X2TextAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("LLMWhisperer")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "llmwhisperer|0a1647f0-f65f-410d-843b-3d979c78350e"

    @staticmethod
    def get_name() -> str:
        return "LLMWhisperer"

    @staticmethod
    def get_description() -> str:
        return "LLMWhisperer X2Text"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/LLMWhisperer.png"

     

    def _get_request_headers(self) -> dict[str, Any]:
        """Obtains the request headers to authenticate with LLMWhisperer.

        Returns:
            str: Request headers
        """
        return {
            "accept": MimeType.JSON,
            WhispererHeader.UNSTRACT_KEY: self.config.get(WhispererConfig.UNSTRACT_KEY),
        }

    def _make_request(
        self,
        request_method: HTTPMethod,
        request_endpoint: str,
        headers: Optional[dict[str, Any]] = None,
        params: Optional[dict[str, Any]] = None,
        data: Optional[Any] = None,
    ) -> Response:
        """Makes a request to LLMWhisperer service.

        Args:
            request_method (HTTPMethod): HTTPMethod to call. Can be GET or POST
            request_endpoint (str): LLMWhisperer endpoint to hit
            headers (Optional[dict[str, Any]], optional): Headers to pass.
                Defaults to None.
            params (Optional[dict[str, Any]], optional): Query params to pass.
                Defaults to None.
            data (Optional[Any], optional): Data to pass in case of POST.
                Defaults to None.

        Returns:
            Response: Response from the request
        """
        llm_whisperer_svc_url = (
            f"{self.config.get(WhispererConfig.URL)}" f"/v1/{request_endpoint}"
        )
        if not headers:
            headers = self._get_request_headers()

        try:
            response: Response
            if request_method == HTTPMethod.GET:
                response = requests.get(
                    url=llm_whisperer_svc_url, headers=headers, params=params
                )
            elif request_method == HTTPMethod.POST:
                response = requests.post(
                    url=llm_whisperer_svc_url,
                    headers=headers,
                    params=params,
                    data=data,
                )
            else:
                raise ExtractorError(f"Unsupported request method: {request_method}")
            response.raise_for_status()
        except ConnectionError as e:
            logger.error(f"Adapter error: {e}")
            raise ExtractorError(
                "Unable to connect to LLMWhisperer service, please check the URL"
            )
        except Timeout as e:
            msg = "Request to LLMWhisperer has timed out"
            logger.error(f"{msg}: {e}")
            raise ExtractorError(msg)
        except HTTPError as e:
            logger.error(f"Adapter error: {e}")
            default_err = "Error while calling the LLMWhisperer service"
            msg = AdapterUtils.get_msg_from_request_exc(
                err=e, message_key="message", default_err=default_err
            )
            raise ExtractorError(msg)
        return response

    def _get_whisper_params(self, enable_highlight: bool = False) -> dict[str, Any]:
        """Gets query params meant for /whisper endpoint.

        The params is filled based on the configuration passed.

        Returns:
            dict[str, Any]: Query params
        """
        params = {
            WhispererConfig.PROCESSING_MODE: self.config.get(
                WhispererConfig.PROCESSING_MODE, ProcessingModes.TEXT.value
            ),
            # Not providing default value to maintain legacy compatablity
            # Providing default value will overide the params
            # processing_mode, force_text_processing
            WhispererConfig.MODE: self.config.get(WhispererConfig.MODE),
            WhispererConfig.OUTPUT_MODE: self.config.get(
                WhispererConfig.OUTPUT_MODE, OutputModes.LINE_PRINTER.value
            ),
            WhispererConfig.FORCE_TEXT_PROCESSING: self.config.get(
                WhispererConfig.FORCE_TEXT_PROCESSING,
                WhispererDefaults.FORCE_TEXT_PROCESSING,
            ),
            WhispererConfig.LINE_SPLITTER_TOLERANCE: self.config.get(
                WhispererConfig.LINE_SPLITTER_TOLERANCE,
                WhispererDefaults.LINE_SPLITTER_TOLERANCE,
            ),
            WhispererConfig.HORIZONTAL_STRETCH_FACTOR: self.config.get(
                WhispererConfig.HORIZONTAL_STRETCH_FACTOR,
                WhispererDefaults.HORIZONTAL_STRETCH_FACTOR,
            ),
            WhispererConfig.PAGES_TO_EXTRACT: self.config.get(
                WhispererConfig.PAGES_TO_EXTRACT,
                WhispererDefaults.PAGES_TO_EXTRACT,
            ),
            WhispererConfig.ADD_LINE_NOS: WhispererDefaults.ADD_LINE_NOS,
            WhispererConfig.OUTPUT_JSON: WhispererDefaults.OUTPUT_JSON,
            WhispererConfig.PAGE_SEPARATOR: self.config.get(
                WhispererConfig.PAGE_SEPARATOR,
                WhispererDefaults.PAGE_SEPARATOR,
            ),
            WhispererConfig.MARK_VERTICAL_LINES: self.config.get(
                WhispererConfig.MARK_VERTICAL_LINES,
                WhispererDefaults.MARK_VERTICAL_LINES,
            ),
            WhispererConfig.MARK_HORIZONTAL_LINES: self.config.get(
                WhispererConfig.MARK_HORIZONTAL_LINES,
                WhispererDefaults.MARK_HORIZONTAL_LINES,
            ),
        }
        if not params[WhispererConfig.FORCE_TEXT_PROCESSING]:
            params.update(
                {
                    WhispererConfig.MEDIAN_FILTER_SIZE: self.config.get(
                        WhispererConfig.MEDIAN_FILTER_SIZE,
                        WhispererDefaults.MEDIAN_FILTER_SIZE,
                    ),
                    WhispererConfig.GAUSSIAN_BLUR_RADIUS: self.config.get(
                        WhispererConfig.GAUSSIAN_BLUR_RADIUS,
                        WhispererDefaults.GAUSSIAN_BLUR_RADIUS,
                    ),
                }
            )

        if enable_highlight:
            params.update(
                {WhispererConfig.STORE_METADATA_FOR_HIGHLIGHTING: enable_highlight}
            )
        return params

    def test_connection(self) -> bool:
        self._make_request(
            request_method=HTTPMethod.GET,
            request_endpoint=WhispererEndpoint.TEST_CONNECTION,
        )
        return True

    def _check_status_until_ready(
        self, whisper_hash: str, headers: dict[str, Any], params: dict[str, Any]
    ) -> WhisperStatus:
        """Checks the extraction status by polling.

        Polls the /whisper-status endpoint in fixed intervals of
        env: ADAPTER_LLMW_POLL_INTERVAL for a certain number of times
        controlled by env: ADAPTER_LLMW_MAX_POLLS.

        Args:
            whisper_hash (str): Identifier for the extraction,
                returned by LLMWhisperer
            headers (dict[str, Any]): Headers to pass for the status check
            params (dict[str, Any]): Params to pass for the status check

        Returns:
            WhisperStatus: Status of the extraction
        """
        POLL_INTERVAL = WhispererDefaults.POLL_INTERVAL
        MAX_POLLS = WhispererDefaults.MAX_POLLS
        request_count = 0

        # Check status in fixed intervals upto max poll count.
        while True:
            request_count += 1
            logger.info(
                f"Checking status with interval: {POLL_INTERVAL}s"
                f", request count: {request_count} [max: {MAX_POLLS}]"
            )
            status_response = self._make_request(
                request_method=HTTPMethod.GET,
                request_endpoint=WhispererEndpoint.STATUS,
                headers=headers,
                params=params,
            )
            if status_response.status_code == 200:
                status_data = status_response.json()
                status = status_data.get(WhisperStatus.STATUS, WhisperStatus.UNKNOWN)
                logger.info(f"Whisper status for {whisper_hash}: {status}")
                if status in [WhisperStatus.PROCESSED, WhisperStatus.DELIVERED]:
                    break
            else:
                raise ExtractorError(
                    "Error checking LLMWhisperer status: "
                    f"{status_response.status_code} - {status_response.text}"
                )

            # Exit with error if max poll count is reached
            if request_count >= MAX_POLLS:
                raise ExtractorError(
                    "Unable to extract text after attempting" f" {request_count} times"
                )
            time.sleep(POLL_INTERVAL)

        return status

    def _extract_async(self, whisper_hash: str) -> str:
        """Makes an async extraction with LLMWhisperer.

        Polls and checks the status first before proceeding to retrieve once.

        Args:
            whisper_hash (str): Identifier of the extraction

        Returns:
            str: Extracted contents from the file
        """
        logger.info(f"Extracting async for whisper hash: {whisper_hash}")

        headers: dict[str, Any] = self._get_request_headers()
        params = {
            WhisperStatus.WHISPER_HASH: whisper_hash,
            WhispererConfig.OUTPUT_JSON: WhispererDefaults.OUTPUT_JSON,
        }

        # Polls in fixed intervals and checks status
        self._check_status_until_ready(
            whisper_hash=whisper_hash, headers=headers, params=params
        )

        retrieve_response = self._make_request(
            request_method=HTTPMethod.GET,
            request_endpoint=WhispererEndpoint.RETRIEVE,
            headers=headers,
            params=params,
        )
        if retrieve_response.status_code == 200:
            return retrieve_response.json()
        else:
            raise ExtractorError(
                "Error retrieving from LLMWhisperer: "
                f"{retrieve_response.status_code} - {retrieve_response.text}"
            )

    def _send_whisper_request(
        self,
        input_file_path: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        enable_highlight: bool = False,
    ) -> requests.Response:
        headers = self._get_request_headers()
        headers["Content-Type"] = "application/octet-stream"
        params = self._get_whisper_params(enable_highlight)

        response: requests.Response
        try:
            response = self._make_request(
                request_method=HTTPMethod.POST,
                request_endpoint=WhispererEndpoint.WHISPER,
                headers=headers,
                params=params,
                data=fs.read(path=input_file_path, mode="rb"),
            )
        except OSError as e:
            logger.error(f"OS error while reading {input_file_path}: {e}")
            raise ExtractorError(str(e))
        return response

    def _extract_text_from_response(
        self,
        output_file_path: Optional[str],
        response: requests.Response,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        output_json = {}
        if response.status_code == 200:
            output_json = response.json()
        elif response.status_code == 202:
            whisper_hash = response.json().get(WhisperStatus.WHISPER_HASH)
            output_json = self._extract_async(whisper_hash=whisper_hash)
        else:
            raise ExtractorError("Couldn't extract text from file")
        if output_file_path:
            self._write_output_to_file(
                output_json=output_json, output_file_path=Path(output_file_path), fs=fs
            )
        return output_json.get("text", "")

    def _write_output_to_file(
        self,
        output_json: dict,
        output_file_path: Path,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> None:
        """Writes the extracted text and metadata to the specified output file
        and metadata file.

        Args:
            output_json (dict): The dictionary containing the extracted data,
                with "text" as the key for the main content.
            output_file_path (Path): The file path where the extracted text
                should be written.

        Raises:
            ExtractorError: If there is an error while writing the output file.
        """
        try:
            text_output = output_json.get("text", "")
            logger.info(f"Writing output to {output_file_path}")
            fs.write(
                path=output_file_path,
                mode="w",
                encoding="utf-8",
                data=text_output,
            )
            try:
                # Define the directory of the output file and metadata paths
                output_dir = output_file_path.parent
                metadata_dir = output_dir / "metadata"
                metadata_file_name = output_file_path.with_suffix(".json").name
                metadata_file_path = metadata_dir / metadata_file_name
                # Ensure the metadata directory exists
                fs.mkdir(str(metadata_dir), create_parents=True)
                # Remove the "text" key from the metadata
                metadata = {
                    key: value for key, value in output_json.items() if key != "text"
                }
                metadata_json = json.dumps(metadata, ensure_ascii=False, indent=4)
                logger.info(f"Writing metadata to {metadata_file_path}")

                fs.write(
                    path=metadata_file_path,
                    mode="w",
                    encoding="utf-8",
                    data=metadata_json,
                )
            except Exception as e:
                logger.error(
                    f"Error while writing metadata to {metadata_file_path}: {e}"
                )

        except Exception as e:
            logger.error(f"Error while writing {output_file_path}: {e}")
            raise ExtractorError(str(e))

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        **kwargs: dict[Any, Any],
    ) -> TextExtractionResult:
        """Used to extract text from documents.

        Args:
            input_file_path (str): Path to file that needs to be extracted
            output_file_path (Optional[str], optional): File path to write
                extracted text into, if None doesn't write to a file.
                Defaults to None.

        Returns:
            str: Extracted text
        """

        response: requests.Response = self._send_whisper_request(
            input_file_path,
            fs,
            bool(kwargs.get(X2TextConstants.ENABLE_HIGHLIGHT, False)),
        )

        metadata = TextExtractionMetadata(
            whisper_hash=response.headers.get(X2TextConstants.WHISPER_HASH, "")
        )

        return TextExtractionResult(
            extracted_text=self._extract_text_from_response(
                output_file_path, response, fs
            ),
            extraction_metadata=metadata,
        )



from .bedrock import BedrockLLM

metadata = {
    "name": BedrockLLM.__name__,
    "version": "1.0.0",
    "adapter": BedrockLLM,
    "description": "Bedrock LLM adapter",
    "is_active": True,
}

__all__ = ["BedrockLLM"]



# isort:skip_file
__all__ = [
    "FileStorage",
    "FileStorageProvider",
    "FileStorageHelper",
    "PermanentFileStorage",
    "SharedTemporaryFileStorage",
    "EnvHelper",
    "StorageType",
]

# Do not change the order of the imports below to avoid circular dependency issues

from unstract.sdk.file_storage.constants import StorageType
from unstract.sdk.file_storage.helper import FileStorageHelper
from unstract.sdk.file_storage.impl import FileStorage
from unstract.sdk.file_storage.permanent import PermanentFileStorage
from unstract.sdk.file_storage.provider import FileStorageProvider
from unstract.sdk.file_storage.shared_temporary import (
    SharedTemporaryFileStorage,
)
from unstract.sdk.file_storage.env_helper import EnvHelper



import sys
from typing import Any

from unstract.sdk.tool.base import BaseTool
from unstract.sdk.tool.entrypoint import ToolEntrypoint


# TODO: Rename tool's class
class ConcreteTool(BaseTool):
    # TODO: Add any checks that need to be done before running the tool
    def validate(self, input_file: str, settings: dict[str, Any]) -> None:
        pass

    def run(
        self,
        settings: dict[str, Any],
        input_file: str,
        output_dir: str,
    ) -> None:
        # -------------- TODO: Add your code here ----------------
        # 1. Read the input_file
        # 2. Process on its contents
        # 3. Write files to the output_dir which need to be copied to the
        #    destination.
        # 4. Write the tool result TEXT or JSON
        # TODO: Write tool result of dict or str
        self.write_tool_result(data={})


if __name__ == "__main__":
    args = sys.argv[1:]
    # TODO: Rename tool's class
    tool = ConcreteTool.from_tool_args(args=args)
    ToolEntrypoint.launch(tool=tool, args=args)



class X2TextConstants:
    PLATFORM_SERVICE_API_KEY = "PLATFORM_SERVICE_API_KEY"
    X2TEXT_HOST = "X2TEXT_HOST"
    X2TEXT_PORT = "X2TEXT_PORT"
    ENABLE_HIGHLIGHT = "enable_highlight"
    TAGS = "tags"
    EXTRACTED_TEXT = "extracted_text"
    WHISPER_HASH = "whisper-hash"
    WHISPER_HASH_V2 = "whisper_hash"



import os
from typing import Any

from llama_index.core.llms import LLM
from llama_index.llms.replicate import Replicate

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter


class Constants:
    MODEL = "model"
    API_KEY = "api_key"


class ReplicateLLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("Replicate")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "replicate|2715ce84-05af-4ab4-b8e9-67ac3211b81e"

    @staticmethod
    def get_name() -> str:
        return "Replicate"

    @staticmethod
    def get_description() -> str:
        return "Replicate LLM"

    @staticmethod
    def get_provider() -> str:
        return "replicate"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/Replicate.png"

     

    @staticmethod
    def can_write() -> bool:
        return True

    @staticmethod
    def can_read() -> bool:
        return True

    def get_llm_instance(self) -> LLM:
        try:
            llm: LLM = Replicate(
                model=str(self.config.get(Constants.MODEL)),
                prompt_key=str(self.config.get(Constants.API_KEY)),
                temperature=0,
            )
            return llm
        except Exception as e:
            raise AdapterError(str(e))



from typing import Any, Union

import requests
from llama_index.core.callbacks import CBEventType, TokenCountingHandler

from unstract.sdk.constants import LogLevel, ToolEnv
from unstract.sdk.helper import SdkHelper
from unstract.sdk.tool.stream import StreamMixin
from unstract.sdk.utils.token_counter import TokenCounter


class Audit(StreamMixin):
    """The 'Audit' class is responsible for pushing usage data to the platform
    service.

    Methods:
        - push_usage_data: Pushes the usage data to the platform service.

    Attributes:
        None
    """

    def __init__(self, log_level: LogLevel = LogLevel.INFO) -> None:
        super().__init__(log_level)

    def push_usage_data(
        self,
        platform_api_key: str,
        token_counter: Union[TokenCountingHandler, TokenCounter] = None,
        model_name: str = "",
        event_type: CBEventType = None,
        kwargs: dict[Any, Any] = None,
    ) -> None:
        """Pushes the usage data to the platform service.

        Args:
            platform_api_key (str): The platform API key.
            token_counter (TokenCountingHandler, optional): The token counter
                object. Defaults to None.
            model_name (str, optional): The name of the model.
                Defaults to "".
            event_type (CBEventType, optional): The type of the event. Defaults
                to None.
            **kwargs: Optional keyword arguments.
                workflow_id (str, optional): The ID of the workflow.
                    Defaults to "".
                execution_id (str, optional): The ID of the execution. Defaults
                    to "".
                adapter_instance_id (str, optional): The adapter instance ID.
                    Defaults to "".
                run_id (str, optional): The run ID. Defaults to "".

        Returns:
            None

        Raises:
            requests.RequestException: If there is an error while pushing the
            usage details.
        """
        platform_host = self.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = self.get_env_or_die(ToolEnv.PLATFORM_PORT)

        base_url = SdkHelper.get_platform_base_url(
            platform_host=platform_host, platform_port=platform_port
        )
        bearer_token = platform_api_key

        workflow_id = kwargs.get("workflow_id", "")
        execution_id = kwargs.get("execution_id", "")
        adapter_instance_id = kwargs.get("adapter_instance_id", "")
        run_id = kwargs.get("run_id", "")
        provider = kwargs.get("provider", "")
        llm_usage_reason = ""
        if event_type == "llm":
            llm_usage_reason = kwargs.get("llm_usage_reason", "")
        data = {
            "workflow_id": workflow_id,
            "execution_id": execution_id,
            "adapter_instance_id": adapter_instance_id,
            "run_id": run_id,
            "usage_type": event_type,
            "llm_usage_reason": llm_usage_reason,
            "model_name": model_name,
            "provider": provider,
            "embedding_tokens": token_counter.total_embedding_token_count,
            "prompt_tokens": token_counter.prompt_llm_token_count,
            "completion_tokens": token_counter.completion_llm_token_count,
            "total_tokens": token_counter.total_llm_token_count,
        }

        url = f"{base_url}/usage"
        headers = {"Authorization": f"Bearer {bearer_token}"}

        try:
            response = requests.post(url, headers=headers, json=data, timeout=30)
            if response.status_code != 200:
                self.stream_log(
                    log=(
                        "Error while pushing usage details: "
                        f"{response.status_code} {response.reason}",
                    ),
                    level=LogLevel.ERROR,
                )
            else:
                self.stream_log(
                    f"Successfully pushed usage details, {data}", level=LogLevel.DEBUG
                )

        except requests.RequestException as e:
            self.stream_log(
                log=f"Error while pushing usage details: {e}",
                level=LogLevel.ERROR,
            )

        finally:
            if isinstance(token_counter, TokenCountingHandler):
                token_counter.reset_counts()

    def push_page_usage_data(
        self,
        platform_api_key: str,
        page_count: int,
        file_size: int,
        file_type: str,
        kwargs: dict[Any, Any] = None,
    ) -> None:
        platform_host = self.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = self.get_env_or_die(ToolEnv.PLATFORM_PORT)
        run_id = kwargs.get("run_id", "")
        file_name = kwargs.get("file_name", "")
        base_url = SdkHelper.get_platform_base_url(
            platform_host=platform_host, platform_port=platform_port
        )
        bearer_token = platform_api_key
        url = f"{base_url}/page-usage"
        headers = {"Authorization": f"Bearer {bearer_token}"}

        data = {
            "page_count": page_count,
            "file_name": file_name,
            "file_size": file_size,
            "file_type": file_type,
            "run_id": run_id,
        }

        try:
            response = requests.post(url, headers=headers, json=data, timeout=30)
            if response.status_code != 200:
                self.stream_log(
                    log=(
                        "Error while pushing page usage details: "
                        f"{response.status_code} {response.reason}",
                    ),
                    level=LogLevel.ERROR,
                )
            else:
                self.stream_log(
                    "Successfully pushed page usage details", level=LogLevel.DEBUG
                )

        except requests.RequestException as e:
            self.stream_log(
                log=f"Error while pushing page usage details: {e}",
                level=LogLevel.ERROR,
            )



import logging
import os
import time
import uuid
from typing import Any

from redis import StrictRedis

logger = logging.getLogger(__name__)


class MetricsMixin:
    TIME_TAKEN_KEY = "time_taken(s)"

    def __init__(self, run_id):
        """Initialize the MetricsMixin class.

        Args:
            run_id (str): Unique identifier for the run.
        """
        self.run_id = run_id
        self.op_id = str(uuid.uuid4())  # Unique identifier for this instance
        self.redis_client = None
        try:
            # Initialize Redis client
            self.redis_client = StrictRedis(
                host=os.getenv("REDIS_HOST", "unstract-redis"),
                port=int(os.getenv("REDIS_PORT", 6379)),
                username=os.getenv("REDIS_USER", "default"),
                password=os.getenv("REDIS_PASSWORD", ""),
                db=1,
                decode_responses=True,
            )
        except Exception as e:
            logger.error(
                "Failed to initialize Redis client" f" for run_id={run_id}: {e}"
            )

        self.redis_key = f"metrics:{self.run_id}:{self.op_id}"

        # Set the start time immediately upon initialization
        self.set_start_time()

    def set_start_time(self, ttl=86400):
        """Store the current timestamp in Redis when the instance is
        created."""
        if self.redis_client is None:
            logger.error("Redis client is not initialized. Cannot set start time.")
            return
        self.redis_client.set(self.redis_key, time.time(), ex=ttl)

    def collect_metrics(self) -> dict[str, Any]:
        """Calculate the time taken since the timestamp was set and delete the
        Redis key.

        Returns:
            dict: The calculated time taken and the associated run_id and op_id.
        """

        if self.redis_client is None:
            logger.error("Redis client is not initialized. Cannot collect metrics.")
            return {self.TIME_TAKEN_KEY: None}

        if not self.redis_client.exists(self.redis_key):
            return {self.TIME_TAKEN_KEY: None}

        start_time = float(self.redis_client.get(self.redis_key))
        time_taken = round(time.time() - start_time, 3)

        # Delete the Redis key after use
        self.redis_client.delete(self.redis_key)

        return {self.TIME_TAKEN_KEY: time_taken}



from .unstructured_community import UnstructuredCommunity

metadata = {
    "name": UnstructuredCommunity.__name__,
    "version": "1.0.0",
    "adapter": UnstructuredCommunity,
    "description": "UnstructuredIO X2Text adapter",
    "is_active": True,
}



import os
from typing import Any, Optional
from urllib.parse import quote_plus

import psycopg2
from llama_index.core.vector_stores.types import BasePydanticVectorStore
from llama_index.vector_stores.postgres import PGVectorStore
from psycopg2._psycopg import connection

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.adapters.vectordb.helper import VectorDBHelper
from unstract.sdk.adapters.vectordb.vectordb_adapter import VectorDBAdapter


class Constants:
    DATABASE = "database"
    HOST = "host"
    PASSWORD = "password"
    PORT = "port"
    USER = "user"
    SCHEMA = "schema"
    ENABLE_SSL = "enable_ssl"


class Postgres(VectorDBAdapter):
    def __init__(self, settings: dict[str, Any]):
        self._config = settings
        self._client: Optional[connection] = None
        self._collection_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        self._schema_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        self._vector_db_instance = self._get_vector_db_instance()
        super().__init__("Postgres", self._vector_db_instance)

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "postgres|70ab6cc2-e86a-4e5a-896f-498a95022d34"

    @staticmethod
    def get_name() -> str:
        return "Postgres"

    @staticmethod
    def get_description() -> str:
        return "Postgres VectorDB"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/postgres.png"

     

    def get_vector_db_instance(self) -> BasePydanticVectorStore:
        return self._vector_db_instance

    def _get_vector_db_instance(self) -> BasePydanticVectorStore:
        try:
            encoded_password = quote_plus(str(self._config.get(Constants.PASSWORD)))
            dimension = self._config.get(
                VectorDbConstants.EMBEDDING_DIMENSION,
                VectorDbConstants.DEFAULT_EMBEDDING_SIZE,
            )
            self._collection_name = VectorDBHelper.get_collection_name(
                self._config.get(VectorDbConstants.VECTOR_DB_NAME),
                dimension,
            )
            self._schema_name = self._config.get(
                Constants.SCHEMA,
                VectorDbConstants.DEFAULT_VECTOR_DB_NAME,
            )
            vector_db: BasePydanticVectorStore = PGVectorStore.from_params(
                database=self._config.get(Constants.DATABASE),
                schema_name=self._schema_name,
                host=self._config.get(Constants.HOST),
                password=encoded_password,
                port=str(self._config.get(Constants.PORT)),
                user=self._config.get(Constants.USER),
                table_name=self._collection_name,
                embed_dim=dimension,
            )
            if self._config.get(Constants.ENABLE_SSL, True):
                ssl_mode = "require"
            else:
                ssl_mode = "disable"
            self._client = psycopg2.connect(
                database=self._config.get(Constants.DATABASE),
                host=self._config.get(Constants.HOST),
                user=self._config.get(Constants.USER),
                password=self._config.get(Constants.PASSWORD),
                port=str(self._config.get(Constants.PORT)),
                sslmode=ssl_mode,
            )

            return vector_db
        except Exception as e:
            raise AdapterError(str(e))

    def test_connection(self) -> bool:
        vector_db = self.get_vector_db_instance()
        test_result: bool = VectorDBHelper.test_vector_db_instance(
            vector_store=vector_db
        )

        # Delete the collection that was created for testing
        if self._client is not None:
            self._client.cursor().execute(
                f"DROP TABLE IF EXISTS "
                f"{self._schema_name}.data_{self._collection_name} CASCADE"
            )
            self._client.commit()

        return test_result

    def close(self, **kwargs: Any) -> None:
        if self._client:
            self._client.close()



from .ollama import OllamaLLM

metadata = {
    "name": OllamaLLM.__name__,
    "version": "1.0.0",
    "adapter": OllamaLLM,
    "description": "Ollama LLM adapter",
    "is_active": True,
}

__all__ = ["OllamaLLM"]



import os
from typing import Any

import httpx
from llama_index.core.embeddings import BaseEmbedding
from llama_index.embeddings.openai import OpenAIEmbedding

from unstract.sdk.adapters.embedding.embedding_adapter import EmbeddingAdapter
from unstract.sdk.adapters.embedding.helper import EmbeddingHelper
from unstract.sdk.adapters.exceptions import AdapterError


class Constants:
    API_KEY = "api_key"
    MODEL = "model"
    API_BASE_VALUE = "https://api.openai.com/v1/"
    API_BASE_KEY = "api_base"
    ADAPTER_NAME = "adapter_name"
    API_TYPE = "openai"
    TIMEOUT = "timeout"
    DEFAULT_TIMEOUT = 240
    DEFAULT_MODEL = "text-embedding-ada-002"


class OpenAI(EmbeddingAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("OpenAI")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "openai|717a0b0e-3bbc-41dc-9f0c-5689437a1151"

    @staticmethod
    def get_name() -> str:
        return "OpenAI"

    @staticmethod
    def get_description() -> str:
        return "OpenAI LLM"

    @staticmethod
    def get_provider() -> str:
        return "openai"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/OpenAI.png"

    def get_embedding_instance(self) -> BaseEmbedding:
        try:
            timeout = int(self.config.get(Constants.TIMEOUT, Constants.DEFAULT_TIMEOUT))
            httpx_timeout = httpx.Timeout(10.0, connect=60.0)
            httpx_client = httpx.Client(timeout=httpx_timeout)
            embedding: BaseEmbedding = OpenAIEmbedding(
                api_key=str(self.config.get(Constants.API_KEY)),
                api_base=str(
                    self.config.get(Constants.API_BASE_KEY, Constants.API_BASE_VALUE)
                ),
                model=str(self.config.get(Constants.MODEL, Constants.DEFAULT_MODEL)),
                api_type=Constants.API_TYPE,
                timeout=timeout,
                http_client=httpx_client,
            )
            return embedding
        except Exception as e:
            raise AdapterError(str(e))

     



import logging
import os
from typing import Any, Optional

from unstract.sdk.adapters.x2text.dto import TextExtractionResult
from unstract.sdk.adapters.x2text.helper import UnstructuredHelper
from unstract.sdk.adapters.x2text.x2text_adapter import X2TextAdapter
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class UnstructuredCommunity(X2TextAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("UnstructuredIOCommunity")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "unstructuredcommunity|eeed506f-1875-457f-9101-846fc7115676"

    @staticmethod
    def get_name() -> str:
        return "Unstructured IO Community"

    @staticmethod
    def get_description() -> str:
        return "Unstructured IO Community X2Text"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/UnstructuredIO.png"

     

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        **kwargs: dict[Any, Any],
    ) -> TextExtractionResult:
        extracted_text: str = UnstructuredHelper.process_document(
            self.config, input_file_path, output_file_path, fs
        )

        return TextExtractionResult(extracted_text=extracted_text)

    def test_connection(self) -> bool:
        result: bool = UnstructuredHelper.test_server_connection(self.config)
        return result



import os
from typing import Any

from llama_index.core.embeddings import BaseEmbedding
from llama_index.embeddings.ollama import OllamaEmbedding

from unstract.sdk.adapters.embedding.embedding_adapter import EmbeddingAdapter
from unstract.sdk.adapters.embedding.helper import EmbeddingHelper
from unstract.sdk.adapters.exceptions import AdapterError


class Constants:
    MODEL = "model_name"
    ADAPTER_NAME = "adapter_name"
    BASE_URL = "base_url"


class Ollama(EmbeddingAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("Ollama")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "ollama|d58d7080-55a9-4542-becd-8433528e127b"

    @staticmethod
    def get_name() -> str:
        return "Ollama"

    @staticmethod
    def get_description() -> str:
        return "Ollama Embedding"

    @staticmethod
    def get_provider() -> str:
        return "ollama"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/ollama.png"

    def get_embedding_instance(self) -> BaseEmbedding:
        try:
            embedding_batch_size = EmbeddingHelper.get_embedding_batch_size(
                config=self.config
            )
            embedding: BaseEmbedding = OllamaEmbedding(
                model_name=str(self.config.get(Constants.MODEL)),
                base_url=str(self.config.get(Constants.BASE_URL)),
                embed_batch_size=embedding_batch_size,
            )
            return embedding
        except Exception as e:
            raise AdapterError(str(e))

     



from .llama_parse import LlamaParseAdapter

metadata = {
    "name": LlamaParseAdapter.__name__,
    "version": "1.0.0",
    "adapter": LlamaParseAdapter,
    "description": "LlamaParse X2Text adapter",
    "is_active": True,
}



from unstract.sdk.exceptions import SdkError


# TODO: Remove redundant AdapterError classes and use SdkError itself
class AdapterError(SdkError):
    pass


class LLMError(AdapterError):
    pass


class ExtractorError(AdapterError):
    pass



import json
import os.path
from json import JSONDecodeError

import pytest
from dotenv import load_dotenv

from unstract.sdk.file_storage import FileStorageProvider, PermanentFileStorage

load_dotenv()


class TEST_CONSTANTS:
    READ_FOLDER_PATH = os.environ.get("READ_FOLDER_PATH")
    WRITE_FOLDER_PATH = os.environ.get("WRITE_FOLDER_PATH")
    RECURSION_FOLDER_PATH = os.environ.get("RECURSION_FOLDER_PATH")
    READ_PDF_FILE = os.environ.get("READ_PDF_FILE")
    READ_TEXT_FILE = os.environ.get("READ_TEXT_FILE")
    WRITE_PDF_FILE = os.environ.get("WRITE_PDF_FILE")
    WRITE_TEXT_FILE = os.environ.get("WRITE_TEXT_FILE")
    TEST_FOLDER = os.environ.get("TEST_FOLDER")
    GCS_BUCKET = os.environ.get("GCS_BUCKET")
    TEXT_CONTENT = os.environ.get("TEXT_CONTENT")
    FILE_STORAGE_GCS = "FILE_STORAGE_GCS"
    FILE_STORAGE_MINIO = "FILE_STORAGE_MINIO"
    FILE_STORAGE_LOCAL = "FILE_STORAGE_LOCAL"


def permanent_file_storage(provider: FileStorageProvider):
    try:
        if provider == FileStorageProvider.GCS:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_GCS, "{}"))
        elif provider == FileStorageProvider.LOCAL:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_LOCAL, "{}"))
        elif provider == FileStorageProvider.MINIO:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_MINIO, "{}"))
    except JSONDecodeError:
        creds = {}
    file_storage = PermanentFileStorage(provider=provider, **creds)
    assert file_storage is not None
    return file_storage


@pytest.mark.parametrize(
    "file_storage, file_read_path, read_mode",
    [
        (
            permanent_file_storage(provider=FileStorageProvider.GCS),
            "fsspec-test/input/3.txt",
            "r",
        ),
        (
            permanent_file_storage(provider=FileStorageProvider.MINIO),
            "fsspec-test/input/3.txt",
            "r",
        ),
        (
            permanent_file_storage(provider=FileStorageProvider.AZURE),
            "fsspec-test/input/3.txt",
            "r",
        ),
    ],
)
def test_permanent_fs_copy_on_read(file_storage, file_read_path, read_mode):
    if file_storage.exists(file_read_path):
        file_storage.rm(file_read_path)
    with pytest.raises(FileNotFoundError):
        file_storage.read(
            file_read_path,
            read_mode,
        )


@pytest.mark.parametrize(
    "file_storage, file_read_path, read_mode, legacy_storage_path, "
    "file_write_path, write_mode",
    [
        (
            permanent_file_storage(provider=FileStorageProvider.GCS),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/legacy_storage/3.txt",
            "fsspec-test/output/copy_on_read_legacy_storage.txt",
            "w",
        ),
        (
            permanent_file_storage(provider=FileStorageProvider.MINIO),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/legacy_storage/3.txt",
            "fsspec-test/output/copy_on_read_legacy_storage.txt",
            "w",
        ),
        (
            permanent_file_storage(provider=FileStorageProvider.AZURE),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/legacy_storage/3.txt",
            "fsspec-test/output/copy_on_read_legacy_storage.txt",
            "w",
        ),
    ],
)
def test_permanent_fs_copy_on_read_with_legacy_storage(
    file_storage,
    file_read_path,
    read_mode,
    legacy_storage_path,
    file_write_path,
    write_mode,
):
    if file_storage.exists(file_read_path):
        file_storage.rm(file_read_path)
    file_read_contents = file_storage.read(
        file_read_path, read_mode, legacy_storage_path=legacy_storage_path
    )
    print(file_read_contents)
    if file_storage.exists(file_write_path):
        file_storage.rm(file_write_path)
    file_storage.write(file_write_path, write_mode, data=file_read_contents)

    file_write_contents = file_storage.read(file_write_path, read_mode)
    assert len(file_read_contents) == len(file_write_contents)


@pytest.mark.parametrize(
    "file_storage, file_read_path, read_mode, file_write_path, write_mode",
    [
        (
            permanent_file_storage(provider=FileStorageProvider.LOCAL),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/output/copy_on_write.txt",
            "w",
        ),
    ],
)
def test_permanent_fs_copy(
    file_storage, file_read_path, read_mode, file_write_path, write_mode
):
    file_read_contents = file_storage.read(file_read_path, read_mode)
    print(file_read_contents)
    if file_storage.exists(file_write_path):
        file_storage.rm(file_write_path)
    file_storage.write(file_write_path, write_mode, data=file_read_contents)
    file_write_contents = file_storage.read(file_write_path, read_mode)
    assert len(file_read_contents) == len(file_write_contents)


@pytest.mark.parametrize(
    "file_storage, from_path, read_mode, to_path, write_mode",
    [
        (
            permanent_file_storage(provider=FileStorageProvider.GCS),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/output/test_write.txt",
            "w",
        ),
        (
            permanent_file_storage(provider=FileStorageProvider.MINIO),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/output/test_write.txt",
            "w",
        ),
        (
            permanent_file_storage(provider=FileStorageProvider.AZURE),
            "fsspec-test/input/3.txt",
            "r",
            "fsspec-test/output/test_write.txt",
            "w",
        ),
    ],
)
def test_permanent_fs_download(file_storage, from_path, read_mode, to_path, write_mode):
    file_read_contents = file_storage.read(from_path, read_mode)
    print(file_read_contents)
    file_storage.download(from_path, to_path)
    local_file_storage = permanent_file_storage(provider=FileStorageProvider.LOCAL)
    file_write_contents = local_file_storage.read(to_path, read_mode)
    assert len(file_read_contents) == len(file_write_contents)


@pytest.mark.parametrize(
    "provider",
    [(FileStorageProvider.GCS), (FileStorageProvider.LOCAL)],
)
def test_permanent_supported_file_storage_mode(provider):
    file_storage = permanent_file_storage(provider=provider)
    assert file_storage is not None and isinstance(file_storage, PermanentFileStorage)



import unittest
from typing import Any

from unstract.sdk.cache import ToolCache
from unstract.sdk.tool.base import BaseTool


# Requires the platform service to be run and
# PLATFORM_SERVICE_API_KEY env to be set
class UnstractToolCacheTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            # self.stream_log("Mock tool running")
            pass

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool()

    def test_set(self):
        cache = ToolCache(
            tool=self.tool, platform_host="http://localhost", platform_port=3001
        )
        result = cache.set(key="test_key", value="test_value")
        self.assertTrue(result)

    def test_get(self):
        cache = ToolCache(
            tool=self.tool, platform_host="http://localhost", platform_port=3001
        )
        cache.set(key="test_key", value="test_value")
        result = cache.get(key="test_key")
        self.assertEqual(result, "test_value")

    def test_delete(self):
        cache = ToolCache(
            tool=self.tool, platform_host="http://localhost", platform_port=3001
        )
        cache.set(key="test_key", value="test_value")
        result = cache.delete(key="test_key")
        self.assertTrue(result)
        result = cache.get(key="test_key")
        self.assertIsNone(result)


if __name__ == "__main__":
    unittest.main()



from .google_document_ai import GoogleDocumentAI

metadata = {
    "name": GoogleDocumentAI.__name__,
    "version": "1.0.0",
    "adapter": GoogleDocumentAI,
    "description": "Google Document AI OCR adapter",
    "is_active": True,
}



from anthropic import APIError as AnthropicAPIError
from google.api_core.exceptions import GoogleAPICallError
from mistralai.models import SDKError as MistralError
from openai import APIError as OpenAIAPIError
from vertexai.generative_models import ResponseValidationError

from unstract.sdk.adapters.exceptions import LLMError
from unstract.sdk.adapters.llm.anthropic.src import AnthropicLLM
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter
from unstract.sdk.adapters.llm.mistral.src import MistralLLM
from unstract.sdk.adapters.llm.open_ai.src import OpenAILLM
from unstract.sdk.adapters.llm.palm.src import PaLMLLM
from unstract.sdk.adapters.llm.vertex_ai.src import VertexAILLM


def parse_llm_err(e: Exception, llm_adapter: LLMAdapter) -> LLMError:
    """Parses the exception from LLM provider.

    Helps parse the LLM error and wraps it with our
    custom exception object to contain a user friendly message.

    Args:
        e (Exception): Error from LLM provider

    Returns:
        LLMError: Unstract's LLMError object
    """
    # Avoid wrapping LLMError objects again
    if isinstance(e, LLMError):
        return e

    if isinstance(e, ResponseValidationError):
        err = VertexAILLM.parse_llm_err(e)
    elif isinstance(e, OpenAIAPIError):
        err = OpenAILLM.parse_llm_err(e)
    elif isinstance(e, AnthropicAPIError):
        err = AnthropicLLM.parse_llm_err(e)
    elif isinstance(e, MistralError):
        err = MistralLLM.parse_llm_err(e)
    elif isinstance(e, GoogleAPICallError):
        err = PaLMLLM.parse_llm_err(e)
    else:
        err = LLMError(str(e), actual_err=e)

    msg = f"Error from LLM provider '{llm_adapter.get_name()}'."

    # Add a code block only for errors from clients
    if err.actual_err:
        msg += f"\n```\n{str(err)}\n```"
    else:
        msg += str(err)
    err.message = msg
    return err



from dataclasses import dataclass
from typing import Optional


@dataclass
class WhispererRequestParams:
    """DTO for LLM Whisperer API request parameters.

    Args:
        tag: Tag value. Can be initialized with List[str], str, or None.
             Will be converted to str | None after initialization.
    """

    # TODO: Extend this DTO to include all Whisperer API parameters
    tag: Optional[str] = None

    def __post_init__(self) -> None:
        # TODO: Allow list of tags once its supported in LLMW v2
        if isinstance(self.tag, list):
            self.tag = self.tag[0] if self.tag else None



import functools
import logging
import time
import uuid

from unstract.sdk.constants import LogLevel
from unstract.sdk.metrics_mixin import MetricsMixin

logger = logging.getLogger(__name__)


class CommonUtils:
    @staticmethod
    def generate_uuid() -> str:
        """Class method to get uuid."""
        return str(uuid.uuid4())


# Mapping from python log level to Unstract counterpart
PY_TO_UNSTRACT_LOG_LEVEL = {
    logging.DEBUG: LogLevel.DEBUG,
    logging.INFO: LogLevel.INFO,
    logging.WARNING: LogLevel.WARN,
    logging.ERROR: LogLevel.ERROR,
}

# Mapping from Unstract log level to python counterpart
UNSTRACT_TO_PY_LOG_LEVEL = {
    LogLevel.DEBUG: logging.DEBUG,
    LogLevel.INFO: logging.INFO,
    LogLevel.WARN: logging.WARNING,
    LogLevel.ERROR: logging.ERROR,
}


def log_elapsed(operation):
    """Adds an elapsed time log.

    Args:
        operation (str): Operation being measured
    """

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            start_time = time.time()
            try:
                result = func(*args, **kwargs)
            finally:
                end_time = time.time()
                elapsed_time = end_time - start_time
                logger.info(f"Time taken for '{operation}': {elapsed_time:.3f}s")
            return result

        return wrapper

    return decorator


def capture_metrics(func):
    """Decorator to capture metrics at the start and end of a function."""

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        # Ensure the required attributes exist; if not,
        # execute the function and return its result
        if not all(
            hasattr(self, attr) for attr in ["_run_id", "_capture_metrics", "_metrics"]
        ):
            return func(self, *args, **kwargs)

        # Check if run_id exists and if metrics should be captured
        metrics_mixin = None
        time_taken_key = MetricsMixin.TIME_TAKEN_KEY
        if self._run_id and self._capture_metrics:
            metrics_mixin = MetricsMixin(run_id=self._run_id)

        try:
            result = func(self, *args, **kwargs)
        finally:
            # If metrics are being captured, collect and assign them at the end
            if metrics_mixin:
                new_metrics = metrics_mixin.collect_metrics()

                # If time_taken(s) exists in both self._metrics and new_metrics, sum it
                if (
                    self._metrics
                    and time_taken_key in self._metrics
                    and time_taken_key in new_metrics
                ):
                    previously_measured_time = self._metrics.get(time_taken_key)
                    newly_measured_time = new_metrics.get(time_taken_key)

                    # Only sum if both are valid
                    if previously_measured_time and newly_measured_time:
                        self._metrics[time_taken_key] = (
                            previously_measured_time + newly_measured_time
                        )
                    else:
                        self._metrics[time_taken_key] = None
                else:
                    # If the key isn't in self._metrics, set it to new_metrics
                    self._metrics = new_metrics

        return result

    return wrapper



import datetime
import os
from abc import ABC, abstractmethod
from json import JSONDecodeError
from pathlib import Path
from typing import Any, Union

from unstract.sdk.constants import (
    Command,
    LogLevel,
    MetadataKey,
    PropKey,
    ToolEnv,
    ToolExecKey,
)
from unstract.sdk.exceptions import FileStorageError
from unstract.sdk.file_storage import EnvHelper, StorageType
from unstract.sdk.tool.mixin import ToolConfigHelper
from unstract.sdk.tool.parser import ToolArgsParser
from unstract.sdk.tool.stream import StreamMixin
from unstract.sdk.utils import ToolUtils


class BaseTool(ABC, StreamMixin):
    """Abstract class for Unstract tools."""

    def __init__(self, log_level: LogLevel = LogLevel.INFO) -> None:
        """Creates an UnstractTool.

        Args:
            log_level (str): Log level for the tool
                Can be one of INFO, DEBUG, WARN, ERROR, FATAL.
        """
        self.start_time = datetime.datetime.now()
        super().__init__(log_level=log_level)
        self.properties = ToolConfigHelper.properties()
        self.spec = ToolConfigHelper.spec()
        self.variables = ToolConfigHelper.variables()
        self.workflow_id = ""
        self.execution_id = ""
        self.file_execution_id = ""
        self.tags = []
        self.source_file_name = ""
        self.org_id = ""
        self._exec_metadata = {}
        self.filestorage_provider = None
        self.workflow_filestorage = None
        self.execution_dir = None
        filestorage_env = os.environ.get(
            ToolEnv.WORKFLOW_EXECUTION_FILE_STORAGE_CREDENTIALS
        )
        if filestorage_env:
            self.execution_dir = Path(self.get_env_or_die(ToolEnv.EXECUTION_DATA_DIR))

            try:
                self.workflow_filestorage = EnvHelper.get_storage(
                    StorageType.SHARED_TEMPORARY,
                    ToolEnv.WORKFLOW_EXECUTION_FILE_STORAGE_CREDENTIALS,
                )
            except KeyError as e:
                self.stream_error_and_exit(
                    f"Required credentials is missing in the env: {str(e)}"
                )
            except FileStorageError as e:
                self.stream_error_and_exit(
                    "Error while initialising storage: %s",
                    e,
                    stack_info=True,
                    exc_info=True,
                )

    @classmethod
    def from_tool_args(cls, args: list[str]) -> "BaseTool":
        """Builder method to create a tool from args passed to a tool.

        Refer the tool's README to know more about the possible args

        Args:
            args (List[str]): Arguments passed to a tool

        Returns:
            AbstractTool: Abstract base tool class
        """
        parsed_args = ToolArgsParser.parse_args(args)
        tool = cls(log_level=parsed_args.log_level)
        if parsed_args.command not in Command.static_commands():
            tool._exec_metadata = tool._get_exec_metadata()
            tool.workflow_id = tool._exec_metadata.get(MetadataKey.WORKFLOW_ID)
            tool.execution_id = tool._exec_metadata.get(MetadataKey.EXECUTION_ID, "")
            tool.file_execution_id = tool._exec_metadata.get(
                MetadataKey.FILE_EXECUTION_ID, ""
            )
            tool.tags = tool._exec_metadata.get(MetadataKey.TAGS, [])
            tool.source_file_name = tool._exec_metadata.get(MetadataKey.SOURCE_NAME, "")
            tool.org_id = tool._exec_metadata.get(MetadataKey.ORG_ID)
        return tool

    def elapsed_time(self) -> float:
        """Returns the elapsed time since the tool was created."""
        return (datetime.datetime.now() - self.start_time).total_seconds()

    def handle_static_command(self, command: str) -> None:
        """Handles a static command.

        Used to handle commands that do not require any processing. Currently,
        the only supported static commands are
        SPEC, PROPERTIES, VARIABLES and ICON.

        This is used by the Unstract SDK to handle static commands.
        It is not intended to be used by the tool. The tool
        stub will automatically handle static commands.

        Args:
            command (str): The static command.
        Returns:
            None
        """
        if command == Command.SPEC:
            self.stream_spec(ToolUtils.json_to_str(self.spec))
        elif command == Command.PROPERTIES:
            self.stream_properties(ToolUtils.json_to_str(self.properties))
        elif command == Command.ICON:
            self.stream_icon(ToolConfigHelper.icon())
        elif command == Command.VARIABLES:
            self.stream_variables(ToolUtils.json_to_str(self.variables))
        else:
            raise ValueError(f"Unknown command {command}")

    def _get_file_from_data_dir(self, file_to_get: str, raise_err: bool = False) -> str:
        base_path = self.execution_dir
        file_path = base_path / file_to_get
        if raise_err and not self.workflow_filestorage.exists(path=file_path):
            self.stream_error_and_exit(
                f"{file_to_get} is missing in EXECUTION_DATA_DIR"
            )

        return str(file_path)

    def get_source_file(self) -> str:
        """Gets the absolute path to the workflow execution's input file
        (SOURCE).

        Returns:
            str: Absolute path to the source file
        """
        return self._get_file_from_data_dir(ToolExecKey.SOURCE, raise_err=True)

    def get_input_file(self) -> str:
        """Gets the absolute path to the input file that's meant for the tool
        being run (INFILE).

        Returns:
            str: Absolute path to the input file
        """
        return self._get_file_from_data_dir(ToolExecKey.INFILE, raise_err=True)

    def get_output_dir(self) -> str:
        """Get the absolute path to the output folder where the tool needs to
        place its output file. This is where the tool writes its output files
        that need to be copied into the destination (COPY_TO_FOLDER path).

        Returns:
            str: Absolute path to the output directory.
        """
        base_path = self.execution_dir
        return str(base_path / ToolExecKey.OUTPUT_DIR)

    @property
    def get_exec_metadata(self) -> dict[str, Any]:
        """Getter for `exec_metadata` of the tool.

        Returns:
            dict[str, Any]: Contents of METADATA.json
        """
        return self._exec_metadata

    def _get_exec_metadata(self) -> dict[str, Any]:
        """Retrieve the contents from METADATA.json present in the data
        directory. This file contains metadata for the tool execution.

        Returns:
            dict[str, Any]: Contents of METADATA.json
        """
        base_path = self.execution_dir
        metadata_path = base_path / ToolExecKey.METADATA_FILE
        metadata_json = {}
        try:
            metadata_json = ToolUtils.load_json(
                file_to_load=metadata_path, fs=self.workflow_filestorage
            )
        except JSONDecodeError as e:
            self.stream_error_and_exit(f"JSON decode error for {metadata_path}: {e}")
        except FileNotFoundError:
            self.stream_error_and_exit(f"Metadata file not found at {metadata_path}")
        except OSError as e:
            self.stream_error_and_exit(f"OS Error while opening {metadata_path}: {e}")
        return metadata_json

    def _write_exec_metadata(self, metadata: dict[str, Any]) -> None:
        """Helps write the `METADATA.JSON` file.

        Args:
            metadata (dict[str, Any]): Metadata to write
        """
        base_path = self.execution_dir
        metadata_path = base_path / ToolExecKey.METADATA_FILE
        ToolUtils.dump_json(
            file_to_dump=metadata_path,
            json_to_dump=metadata,
            fs=self.workflow_filestorage,
        )

    def _update_exec_metadata(self) -> None:
        """Updates the execution metadata after a tool executes.

        Currently overrwrites most of the keys with the latest tool
        executed.
        """
        tool_metadata = {
            MetadataKey.TOOL_NAME: self.properties[PropKey.FUNCTION_NAME],
            MetadataKey.ELAPSED_TIME: self.elapsed_time(),
            MetadataKey.OUTPUT_TYPE: self.properties[PropKey.RESULT][PropKey.TYPE],
        }
        if MetadataKey.TOTAL_ELA_TIME not in self._exec_metadata:
            self._exec_metadata[MetadataKey.TOTAL_ELA_TIME] = self.elapsed_time()
        else:
            self._exec_metadata[MetadataKey.TOTAL_ELA_TIME] += self.elapsed_time()

        if MetadataKey.TOOL_META not in self._exec_metadata:
            self._exec_metadata[MetadataKey.TOOL_META] = [tool_metadata]
        else:
            self._exec_metadata[MetadataKey.TOOL_META].append(tool_metadata)

        self._write_exec_metadata(metadata=self._exec_metadata)

    def update_exec_metadata(self, metadata: dict[str, Any]) -> None:
        """Helps update the execution metadata with the provided metadata
        dictionary.

        This method iterates over the key-value pairs in the input metadata dictionary
        and updates the internal `_exec_metadata` dictionary of the tool instance
        accordingly. It then writes the updated metadata to the `METADATA.json`
        file in the tool's data directory.

        Args:
            metadata (dict[str, Any]): A dictionary containing the metadata
            key-value pairs to update in the execution metadata.

        Returns:
            None
        """
        for key, value in metadata.items():
            self._exec_metadata[key] = value

        self._write_exec_metadata(metadata=self._exec_metadata)

    def write_tool_result(self, data: Union[str, dict[str, Any]]) -> None:
        """Helps write contents of the tool result into TOOL_DATA_DIR.

        Args:
            data (Union[str, dict[str, Any]]): Data to be written
        """
        output_type = self.properties[PropKey.RESULT][PropKey.TYPE]
        if output_type is PropKey.OutputType.JSON and not isinstance(data, dict):
            # TODO: Validate JSON type output with output schema as well
            self.stream_error_and_exit(
                f"Expected result to have type {PropKey.OutputType.JSON} "
                f"but {type(data)} is passed"
            )

        # TODO: Review if below is necessary
        result = {
            "workflow_id": self.workflow_id,
            "elapsed_time": self.elapsed_time(),
            "output": data,
        }
        self.stream_result(result)

        self._update_exec_metadata()
        # INFILE is overwritten for next tool to run
        input_file_path: Path = Path(self.get_input_file())
        ToolUtils.dump_json(
            file_to_dump=input_file_path,
            json_to_dump=data,
            fs=self.workflow_filestorage,
        )

    def validate(self, input_file: str, settings: dict[str, Any]) -> None:
        """Override to implement custom validation for the tool.

        Args:
            input_file (str): Path to the file that will be used for execution.
            settings (dict[str, Any]): Settings configured for the tool.
                Defaults initialized through the tool's SPEC.
        """
        pass

    @abstractmethod
    def run(
        self,
        settings: dict[str, Any],
        input_file: str,
        output_dir: str,
    ) -> None:
        """Implements RUN command for the tool.

        Args:
            settings (dict[str, Any]): Settings for the tool
            input_file (str): Path to the input file
            output_dir (str): Path to write the tool output to which gets copied
                into the destination
        """
        pass



import json
import logging
import os
from typing import Any, Optional

from google.auth.transport import requests as google_requests
from google.oauth2.service_account import Credentials
from llama_index.core.llms import LLM
from llama_index.llms.vertex import Vertex
from vertexai.generative_models import Candidate, FinishReason, ResponseValidationError
from vertexai.generative_models._generative_models import (
    HarmBlockThreshold,
    HarmCategory,
)

from unstract.sdk.adapters.exceptions import LLMError
from unstract.sdk.adapters.llm.constants import LLMKeys
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter

logger = logging.getLogger(__name__)


class Constants:
    MODEL = "model"
    PROJECT = "project"
    JSON_CREDENTIALS = "json_credentials"
    MAX_RETRIES = "max_retries"
    MAX_TOKENS = "max_tokens"
    DEFAULT_MAX_TOKENS = 2048
    BLOCK_ONLY_HIGH = "BLOCK_ONLY_HIGH"


class SafetySettingsConstants:
    SAFETY_SETTINGS = "safety_settings"
    DANGEROUS_CONTENT = "dangerous_content"
    HATE_SPEECH = "hate_speech"
    HARASSMENT = "harassment"
    SEXUAL_CONTENT = "sexual_content"
    OTHER = "other"


UNSTRACT_VERTEX_SAFETY_THRESHOLD_MAPPING: dict[str, HarmBlockThreshold] = {
    "HARM_BLOCK_THRESHOLD_UNSPECIFIED": HarmBlockThreshold.HARM_BLOCK_THRESHOLD_UNSPECIFIED,  # noqa: E501
    "BLOCK_LOW_AND_ABOVE": HarmBlockThreshold.BLOCK_LOW_AND_ABOVE,
    "BLOCK_MEDIUM_AND_ABOVE": HarmBlockThreshold.BLOCK_MEDIUM_AND_ABOVE,
    "BLOCK_ONLY_HIGH": HarmBlockThreshold.BLOCK_ONLY_HIGH,
    "BLOCK_NONE": HarmBlockThreshold.BLOCK_NONE,
    "OFF": HarmBlockThreshold.OFF,
}


class VertexAILLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("VertexAILLM")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "vertexai|78fa17a5-a619-47d4-ac6e-3fc1698fdb55"

    @staticmethod
    def get_name() -> str:
        return "VertexAI"

    @staticmethod
    def get_description() -> str:
        return "Vertex Gemini LLM"

    @staticmethod
    def get_provider() -> str:
        return "vertex_ai"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/VertexAI.png"

     

    def get_llm_instance(self) -> LLM:
        input_credentials = self.config.get(Constants.JSON_CREDENTIALS, "{}")
        try:
            json_credentials = json.loads(input_credentials)
        except json.JSONDecodeError:
            raise LLMError(
                "Credentials is not a valid service account JSON, "
                "please provide a valid JSON."
            )

        credentials = Credentials.from_service_account_info(
            info=json_credentials,
            scopes=["https://www.googleapis.com/auth/cloud-platform"],
        )  # type: ignore
        credentials.refresh(google_requests.Request())  # type: ignore
        max_retries = int(
            self.config.get(Constants.MAX_RETRIES, LLMKeys.DEFAULT_MAX_RETRIES)
        )
        max_tokens = int(
            self.config.get(Constants.MAX_TOKENS, Constants.DEFAULT_MAX_TOKENS)
        )

        safety_settings_default_config: dict[str, str] = {
            SafetySettingsConstants.DANGEROUS_CONTENT: Constants.BLOCK_ONLY_HIGH,
            SafetySettingsConstants.HATE_SPEECH: Constants.BLOCK_ONLY_HIGH,
            SafetySettingsConstants.HARASSMENT: Constants.BLOCK_ONLY_HIGH,
            SafetySettingsConstants.SEXUAL_CONTENT: Constants.BLOCK_ONLY_HIGH,
            SafetySettingsConstants.OTHER: Constants.BLOCK_ONLY_HIGH,
        }
        safety_settings_user_config: dict[str, str] = self.config.get(
            SafetySettingsConstants.SAFETY_SETTINGS,
            safety_settings_default_config,
        )

        vertex_safety_settings: dict[
            HarmCategory, HarmBlockThreshold
        ] = self._get_vertex_safety_settings(safety_settings_user_config)

        llm: LLM = Vertex(
            project=str(self.config.get(Constants.PROJECT)),
            model=str(self.config.get(Constants.MODEL)),
            credentials=credentials,
            temperature=0,
            max_retries=max_retries,
            max_tokens=max_tokens,
            safety_settings=vertex_safety_settings,
        )
        return llm

    def _get_vertex_safety_settings(
        self, safety_settings_user_config: dict[str, str]
    ) -> dict[HarmCategory, HarmBlockThreshold]:
        vertex_safety_settings: dict[HarmCategory, HarmBlockThreshold] = dict()
        vertex_safety_settings[
            HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT
        ] = UNSTRACT_VERTEX_SAFETY_THRESHOLD_MAPPING[
            (
                safety_settings_user_config.get(
                    SafetySettingsConstants.DANGEROUS_CONTENT,
                    Constants.BLOCK_ONLY_HIGH,
                )
            )
        ]
        vertex_safety_settings[
            HarmCategory.HARM_CATEGORY_HATE_SPEECH
        ] = UNSTRACT_VERTEX_SAFETY_THRESHOLD_MAPPING[
            (
                safety_settings_user_config.get(
                    SafetySettingsConstants.HATE_SPEECH,
                    Constants.BLOCK_ONLY_HIGH,
                )
            )
        ]
        vertex_safety_settings[
            HarmCategory.HARM_CATEGORY_HARASSMENT
        ] = UNSTRACT_VERTEX_SAFETY_THRESHOLD_MAPPING[
            (
                safety_settings_user_config.get(
                    SafetySettingsConstants.HARASSMENT,
                    Constants.BLOCK_ONLY_HIGH,
                )
            )
        ]
        vertex_safety_settings[
            HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT
        ] = UNSTRACT_VERTEX_SAFETY_THRESHOLD_MAPPING[
            (
                safety_settings_user_config.get(
                    SafetySettingsConstants.SEXUAL_CONTENT,
                    Constants.BLOCK_ONLY_HIGH,
                )
            )
        ]
        vertex_safety_settings[
            HarmCategory.HARM_CATEGORY_UNSPECIFIED
        ] = UNSTRACT_VERTEX_SAFETY_THRESHOLD_MAPPING[
            (
                safety_settings_user_config.get(
                    SafetySettingsConstants.OTHER, Constants.BLOCK_ONLY_HIGH
                )
            )
        ]
        return vertex_safety_settings

    @staticmethod
    def parse_llm_err(e: ResponseValidationError) -> LLMError:
        """Parse the error from Vertex AI.

        Helps parse and raise errors from Vertex AI.
        https://ai.google.dev/api/generate-content#generatecontentresponse

        Args:
            e (ResponseValidationError): Exception from Vertex AI

        Returns:
            LLMError: Error to be sent to the user
        """
        assert len(e.responses) == 1, (
            "Expected e.responses to contain a single element "
            "since its a completion call and not chat."
        )
        resp = e.responses[0]
        candidates: list["Candidate"] = resp.candidates
        if not candidates:
            msg = str(resp.prompt_feedback)
        reason_messages = {
            FinishReason.MAX_TOKENS: (
                "The maximum number of tokens for the LLM has been reached. Please "
                "either tweak your prompts or try using another LLM."
            ),
            FinishReason.STOP: (
                "The LLM stopped generating a response due to the natural stop "
                "point of the model or a provided stop sequence."
            ),
            FinishReason.SAFETY: "The LLM response was flagged for safety reasons.",
            FinishReason.RECITATION: (
                "The LLM response was flagged for recitation reasons."
            ),
            FinishReason.BLOCKLIST: (
                "The LLM response generation was stopped because it "
                "contains forbidden terms."
            ),
            FinishReason.PROHIBITED_CONTENT: (
                "The LLM response generation was stopped because it "
                "potentially contains prohibited content."
            ),
            FinishReason.SPII: (
                "The LLM response generation was stopped because it potentially "
                "contains Sensitive Personally Identifiable Information."
            ),
        }

        reason_status_code = {
            FinishReason.MAX_TOKENS: 429,
            FinishReason.STOP: 200,
            FinishReason.SAFETY: 403,
            FinishReason.RECITATION: 403,
            FinishReason.BLOCKLIST: 403,
            FinishReason.PROHIBITED_CONTENT: 403,
            FinishReason.SPII: 403,
        }

        err_list = []
        status_code: Optional[int] = None
        for candidate in candidates:
            reason: FinishReason = candidate.finish_reason

            if candidate.finish_message:
                err_msg = candidate.finish_message
            else:
                err_msg = reason_messages.get(reason, str(candidate))

            status_code = reason_status_code.get(reason)
            err_list.append(err_msg)
        msg = "\n\nAnother error: \n".join(err_list)
        return LLMError(msg, actual_err=e, status_code=status_code)



import json
import logging
import os

from unstract.sdk.exceptions import FileStorageError
from unstract.sdk.file_storage.constants import CredentialKeyword, StorageType
from unstract.sdk.file_storage.impl import FileStorage
from unstract.sdk.file_storage.permanent import PermanentFileStorage
from unstract.sdk.file_storage.provider import FileStorageProvider
from unstract.sdk.file_storage.shared_temporary import SharedTemporaryFileStorage

logger = logging.getLogger(__name__)


class EnvHelper:
    ENV_CONFIG_FORMAT = (
        '{"provider": "gcs", ' '"credentials": {"token": "/path/to/google/creds.json"}}'
    )

    @staticmethod
    def get_storage(storage_type: StorageType, env_name: str) -> FileStorage:
        """Helper function for clients to pick up remote storage configuration
        from env, initialise the file storage for the same and return the
        instance.

        Args:
            storage_type: Permanent / Temporary file storage
            env_name: Name of the env which has the file storage config

        Returns:
            FileStorage: FIleStorage instance initialised using the provider
            and credentials configured in the env
        """
        try:
            file_storage_creds = json.loads(os.environ.get(env_name, ""))
            provider = FileStorageProvider(
                file_storage_creds[CredentialKeyword.PROVIDER]
            )
            credentials = file_storage_creds.get(CredentialKeyword.CREDENTIALS, {})
            if storage_type == StorageType.PERMANENT:
                file_storage = PermanentFileStorage(provider=provider, **credentials)
            elif storage_type == StorageType.SHARED_TEMPORARY:
                file_storage = SharedTemporaryFileStorage(
                    provider=provider, **credentials
                )
            else:
                raise NotImplementedError()
            return file_storage
        except KeyError as e:
            logger.error(f"Required credentials are missing in the env: {str(e)}")
            logger.error(f"The configuration format is {EnvHelper.ENV_CONFIG_FORMAT}")
            raise e
        except FileStorageError as e:
            raise e



import logging
import os
import time
from typing import Any

from llama_index.core.llms import LLM

from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter
from unstract.sdk.adapters.llm.no_op.src.no_op_custom_llm import NoOpCustomLLM

logger = logging.getLogger(__name__)


class NoOpLLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("NoOpLlm")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "noOpLlm|f673a5a2-90f9-40f5-94c0-9fbc663b7553"

    @staticmethod
    def get_name() -> str:
        return "No Op LLM"

    @staticmethod
    def get_description() -> str:
        return "No Op LLM"

    @staticmethod
    def get_provider() -> str:
        return "noOpLlm"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/noOpLlm.png"

    def get_llm_instance(self) -> LLM:
        llm: LLM = NoOpCustomLLM(wait_time=self.config.get("wait_time"))
        return llm

    def test_connection(self) -> bool:
        llm = self.get_llm_instance()
        if not llm:
            return False
        llm.complete(
            "The capital of Tamilnadu is ",
            temperature=0.003,
        )
        time.sleep(self.config.get("wait_time"))
        return True



import json
import os
import unittest

from dotenv import load_dotenv
from llama_index.core.embeddings import BaseEmbedding
from parameterized import parameterized

from unstract.sdk.embedding import ToolEmbedding
from unstract.sdk.tool.base import BaseTool

load_dotenv()


def get_test_values(env_key: str) -> list[str]:
    test_values = json.loads(os.environ.get(env_key))
    return test_values


class ToolEmbeddingTest(unittest.TestCase):
    TEST_SNIPPET = "Hello, I am Unstract"

    class MockTool(BaseTool):
        def run(
            self,
        ) -> None:
            self.stream_log("Mock tool running")

    def setUp(self) -> None:
        self.tool = self.MockTool()

    def run_embedding_test(self, adapter_instance_id):
        embedding = ToolEmbedding(tool=self.tool)
        embed_model = embedding.get_embedding(adapter_instance_id)
        self.assertIsNotNone(embed_model)
        self.assertIsInstance(embed_model, BaseEmbedding)
        response = embed_model._get_text_embedding(
            ToolEmbeddingTest.TEST_SNIPPET
        )
        self.assertIsNotNone(response)

    @parameterized.expand(get_test_values("EMBEDDING_TEST_VALUES"))
    def test_get_embedding(self, adapter_instance_id: str) -> None:
        self.run_embedding_test(adapter_instance_id)


if __name__ == "__main__":
    unittest.main()



from .vertex_ai import VertexAIEmbedding

metadata = {
    "name": VertexAIEmbedding.__name__,
    "version": "1.0.0",
    "adapter": VertexAIEmbedding,
    "description": "VertexAI embedding adapter",
    "is_active": True,
}



from typing import Any


class ErrorType:
    AUTH_ERR = "authentication_error"
    PERMISSION_ERR = "permission_error"
    NOT_FOUND_ERR = "not_found_error"
    API_ERR = "api_error"
    OVERLOADED_ERR = "overloaded_error"
    RATE_LIMIT_ERR = "rate_limit_error"
    LARGE_REQUEST_ERR = "request_too_large"


def parse_anthropic_err(err_body: dict[str, Any]) -> str:
    """Parses error from Anthropic.

    Refer https://docs.anthropic.com/en/api/errors#http-errors

    Args:
        err_body (dict[str, Any]): Error from Anthropic to parse

    Returns:
        str: Parsed error from Anthropic
    """
    err_type = err_body.get("type")
    msg: str

    if err_type == ErrorType.AUTH_ERR:
        msg = "Authentication error, please check the provided API key"
    elif err_type == ErrorType.PERMISSION_ERR:
        msg = "Your API key does not have permission to use the specified resource"
    elif err_type == ErrorType.NOT_FOUND_ERR:
        err_msg = err_body.get("message")
        msg = f"The requested resource was not found, '{err_msg}'"
    elif err_type == ErrorType.RATE_LIMIT_ERR:
        msg = (
            "Your Anthropic account has hit a rate limit, "
            "please try again after some time"
        )
    elif err_type == ErrorType.LARGE_REQUEST_ERR:
        err_msg = err_body.get("message")
        msg = (
            f"Request exceeded the maximum allowed number of bytes, '{err_msg}'. "
            "Try reducing the number of prompts or reduce the context size by chunking."
        )
    elif err_type == ErrorType.API_ERR:
        msg = "An unexpected error has occurred internal to Anthropic's systems"
    elif err_type == ErrorType.OVERLOADED_ERR:
        msg = (
            "Anthropic's API is temporarily overloaded, please "
            "try again after some time"
        )
    else:
        msg = err_body.get("message", "Anthropic error")
    return msg



import argparse
import logging
import shutil
from json import loads
from pathlib import Path
from typing import Any

from unstract.sdk import get_sdk_version
from unstract.sdk.constants import Command
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.tool.validator import ToolValidator

logger = logging.getLogger(__name__)


class ToolExecutor:
    """Takes care of executing a tool's intended command."""

    def __init__(self, tool: BaseTool) -> None:
        self.tool = tool

    def execute(self, args: argparse.Namespace) -> None:
        """Executes the tool with the passed arguments.

        Args:
            args (argparse.Namespace): Parsed arguments to execute with
        """
        command = str.upper(args.command)

        if command in Command.static_commands():
            self.tool.handle_static_command(command)
        elif command == Command.RUN:
            self.execute_run(args=args)
        else:
            self.tool.stream_error_and_exit(f"Command {command} not supported")

    def _setup_for_run(self) -> None:
        """Helps initialize tool execution for RUN command."""
        shutil.rmtree(self.tool.get_output_dir(), ignore_errors=True)
        Path(self.tool.get_output_dir()).mkdir(parents=True, exist_ok=True)

    def execute_run(self, args: argparse.Namespace) -> None:
        """Executes the tool's RUN command.

        Args:
            args (argparse.Namespace): Parsed arguments to execute with
        """
        if args.settings is None:
            self.tool.stream_error_and_exit("--settings are required for RUN command")
        settings: dict[str, Any] = loads(args.settings)

        tool_name = self.tool.properties["displayName"]
        tool_version = self.tool.properties["toolVersion"]
        self.tool.stream_log(
            f"Running tool '{tool_name}:{tool_version}' with "
            f"Workflow ID: {self.tool.workflow_id}, "
            f"Execution ID: {self.tool.execution_id}, "
            f"SDK Version: {get_sdk_version()}"
        )
        validator = ToolValidator(self.tool)
        settings = validator.validate_pre_execution(settings=settings)

        self.tool.stream_log(
            f"Executing for file: {self.tool.get_exec_metadata['source_name']}, "
            f"with tool settings: {settings}"
        )

        try:
            self.tool.run(
                settings=settings,
                input_file=self.tool.get_input_file(),
                output_dir=self.tool.get_output_dir(),
            )
        except Exception as e:
            msg = f"Error while running tool '{tool_name}': {str(e)}"
            logger.error(msg, stack_info=True, exc_info=True)
            self.tool.stream_error_and_exit(msg)

        # TODO: Call tool method to validate if output was written



import base64
import json
import logging
import os
from typing import Any, Optional

import requests
from filetype import filetype
from google.auth.transport import requests as google_requests
from google.oauth2.service_account import Credentials

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.ocr.constants import FileType
from unstract.sdk.adapters.ocr.ocr_adapter import OCRAdapter
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class GoogleDocumentAIKey:
    RAW_DOCUMENT = "rawDocument"
    MIME_TYPE = "mimeType"
    CONTENT = "content"
    SKIP_HUMAN_REVIEW = "skipHumanReview"
    FIELD_MASK = "fieldMask"


class Constants:
    URL = "url"
    CREDENTIALS = "credentials"
    CREDENTIAL_SCOPES = ["https://www.googleapis.com/auth/cloud-platform"]


class GoogleDocumentAI(OCRAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("GoogleDocumentAI")
        self.config = settings
        google_service_account = self.config.get(Constants.CREDENTIALS)
        if not google_service_account:
            logger.error("Google service account not found")
        else:
            self.google_service_account = json.loads(google_service_account)

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "googledocumentai|1013f64b-ecc9-4e35-b986-aebd60fb55d7"

    @staticmethod
    def get_name() -> str:
        return "GoogleDocumentAI"

    @staticmethod
    def get_description() -> str:
        return "Google Document AI OCR"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/GoogleDocumentAI.png"

     

    """ Construct the request body to be sent to Google AI Document server """

    def _get_request_body(
        self, file_type_mime: str, file_content_in_bytes: bytes
    ) -> dict[str, Any]:
        return {
            GoogleDocumentAIKey.RAW_DOCUMENT: {
                GoogleDocumentAIKey.MIME_TYPE: file_type_mime,
                GoogleDocumentAIKey.CONTENT: base64.b64encode(
                    file_content_in_bytes
                ).decode("utf-8"),
            },
            GoogleDocumentAIKey.SKIP_HUMAN_REVIEW: True,
            GoogleDocumentAIKey.FIELD_MASK: "text",
        }

    """ Construct the request headers to be sent
    to Google AI Document server """

    def _get_request_headers(self) -> dict[str, Any]:
        credentials = Credentials.from_service_account_info(
            self.google_service_account, scopes=Constants.CREDENTIAL_SCOPES
        )  # type: ignore
        credentials.refresh(google_requests.Request())  # type: ignore

        return {
            "Content-Type": "application/json; charset=utf-8",
            "Authorization": f"Bearer {credentials.token}",
        }

    """ Detect the mime type from the file content """

    def _get_input_file_type_mime(
        self,
        input_file_path: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        sample_contents = fs.read(path=input_file_path, mode="rb", length=100)
        file_type = filetype.guess(sample_contents)

        file_type_mime: str = file_type.MIME if file_type else FileType.TEXT_PLAIN

        if file_type_mime not in FileType.ALLOWED_TYPES:
            logger.error("Input file type not supported: " f"{file_type_mime}")

        logger.info(f"file: `{input_file_path} [{file_type_mime}]`\n\n")

        return file_type_mime

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        try:
            file_type_mime = self._get_input_file_type_mime(input_file_path)
            if fs.exists(input_file_path):
                file_content_in_bytes = fs.read(path=input_file_path, mode="rb")
            else:
                raise AdapterError(f"File not found {input_file_path}")
            processor_url = self.config.get(Constants.URL, "") + ":process"
            headers = self._get_request_headers()
            data = self._get_request_body(
                file_type_mime=file_type_mime,
                file_content_in_bytes=file_content_in_bytes,
            )
            response = requests.post(processor_url, headers=headers, json=data)
            if response.status_code != 200:
                logger.error(f"Error while calling Google Document AI: {response.text}")
            response_json: dict[str, Any] = response.json()
            result_text: str = response_json["document"]["text"]
            if output_file_path is not None:
                fs.write(path=output_file_path, mode="w", encoding="utf-8")
            return result_text
        except Exception as e:
            logger.error(f"Error while processing document {e}")
            if not isinstance(e, AdapterError):
                raise AdapterError(str(e))
            else:
                raise e

    def test_connection(self) -> bool:
        try:
            url = self.config.get(Constants.URL, "")
            headers = self._get_request_headers()
            response = requests.get(url, headers=headers)
            if response.status_code != 200:
                logger.error(f"Error while testing Google Document AI: {response.text}")
                raise AdapterError(f"{response.status_code} - {response.reason}")
            else:
                return True
        except Exception as e:
            logger.error(f"Error occured while testing adapter {e}")
            if not isinstance(e, AdapterError):
                raise AdapterError(str(e))
            else:
                raise e



import json
import logging
import os
import unittest
from typing import Any

from dotenv import load_dotenv
from parameterized import parameterized
from unstract.adapters.llm.helper import LLMHelper

from unstract.sdk.llm import ToolLLM
from unstract.sdk.tool.base import BaseTool

load_dotenv()

logger = logging.getLogger(__name__)


def get_test_values(env_key: str) -> list[str]:
    test_values = json.loads(os.environ.get(env_key))
    return test_values


class ToolLLMTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            # self.stream_log("Mock tool running")
            pass

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool()

    @parameterized.expand(
        get_test_values("LLM_TEST_VALUES")
        # AzureOpenAI (Works)
        # OpenAI (Works)
        # AnyScale (llm FAILS)
        # Anthropic (llm.complete FAILS)
        # 1. unsupported params: max_token, stop.
        # TypeError: create() got an unexpected keyword argument
        # 'max_tokens'
        # 2. anthropic.APIConnectionError: Connection error.
        # PaLM (Works)
        # Errors
        # 1. unexpected keyword argument 'max_tokens', 'stop'
        # Replicate (llm.complete FAILS)
        # Errors
        # 1. replicate.exceptions.ReplicateError:
        # You did not pass an authentication token
        # Mistral (llm.complete FAILS)
        # Errors
        # 1.TypeError: chat() got an unexpected keyword argument 'stop'
    )
    def test_get_llm(self, adapter_instance_id):
        tool_llm = ToolLLM(tool=self.tool)
        llm = tool_llm.get_llm(adapter_instance_id)
        self.assertIsNotNone(llm)

        result = LLMHelper.test_llm_instance(llm)
        logger.error(result)
        self.assertEqual(result, True)


if __name__ == "__main__":
    unittest.main()



from .qdrant import Qdrant

metadata = {
    "name": Qdrant.__name__,
    "version": "1.0.0",
    "adapter": Qdrant,
    "description": "Qdrant VectorDB adapter",
    "is_active": True,
}

__all__ = ["Qdrant"]



import json
import logging
import os
import unittest
from typing import Any, Optional
from unittest.mock import Mock, patch

from dotenv import load_dotenv
from parameterized import parameterized

from unstract.sdk.index import Index
from unstract.sdk.tool.base import BaseTool

load_dotenv()

logger = logging.getLogger(__name__)


def get_test_values(env_key: str) -> list[str]:
    test_values = json.loads(os.environ.get(env_key))
    return test_values


class ToolLLMTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            # self.stream_log("Mock tool running")
            pass

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool()

    @patch(
        "unstract.sdk.index.Index.generate_index_key",
        Mock(
            return_value="77843eb8d9e30ad56bfcb018c2633fa32feef2f0c09762b6b820c75664b64c1b"
        ),
    )
    def test_generate_file_id(self):
        expected = "77843eb8d9e30ad56bfcb018c2633fa32feef2f0c09762b6b820c75664b64c1b"
        index = Index(tool=self.tool)
        actual = index.generate_file_id(
            tool_id="8ac26867-7811-4dc7-a17b-b16d3b561583",
            vector_db="81f1f6a8-cae8-4b8e-b2a4-57f80de512f6",
            embedding="ecf998d6-ded0-4aca-acd1-372a21daf0f9",
            x2text="59bc55fc-e2a7-48dd-ae93-794b4d81d46e",
            chunk_size="1024",
            chunk_overlap="128",
            file_path="/a/b/c",
            file_hash="045b1a67824592b67426f8e60c1f8328e8d2a35139f9983e0aa0a7b6f10915c3",
        )
        assert expected == actual

    test_data = [
        {
            "vector_db": "81f1f6a8-cae8-4b8e-b2a4-57f80de512f6",
            "embedding": "ecf998d6-ded0-4aca-acd1-372a21daf0f9",
            "x2text": "59bc55fc-e2a7-48dd-ae93-794b4d81d46e",
            "chunk_size": "1024",
            "chunk_overlap": "128",
            "file_path": "/a/b/c",
        },
        {
            "vector_db": "81f1f6a8-cae8-4b8e-b2a4-57f80de512f6",
            "embedding": "ecf998d6-ded0-4aca-acd1-372a21daf0f9",
            "x2text": "59bc55fc-e2a7-48dd-ae93-794b4d81d46e",
            "chunk_size": "1024",
            "chunk_overlap": "128",
            "file_path": "/a/b/c",
            "file_hash": "045b1a67824592b67426f8e60c1f8328e8d2a35139f9983e0aa0a7b6f10915c3",
        },
    ]

    @parameterized.expand(test_data)
    @patch(
        "unstract.sdk.adapter.ToolAdapter.get_adapter_config",
        Mock(return_value={}),
    )
    @patch(
        "unstract.sdk.utils.ToolUtils.hash_str",
        Mock(
            return_value="77843eb8d9e30ad56bfcb018c2633fa32feef2f0c09762b6b820c75664b64c1b"
        ),
    )
    @patch(
        "unstract.sdk.utils.ToolUtils.get_hash_from_file",
        Mock(
            return_value="ab940bb34a60d2a7876dd8e1bd1b22f5dc85936a9e2af3c49bfc888a1d944ff0"
        ),
    )
    def test_generate_index_key(
        self,
        vector_db: str,
        embedding: str,
        x2text: str,
        chunk_size: str,
        chunk_overlap: str,
        file_path: Optional[str] = None,
        file_hash: Optional[str] = None,
    ):
        expected = "77843eb8d9e30ad56bfcb018c2633fa32feef2f0c09762b6b820c75664b64c1b"
        index = Index(tool=self.tool)
        actual = index.generate_index_key(
            vector_db,
            embedding,
            x2text,
            chunk_size,
            chunk_overlap,
            file_path,
            file_hash,
        )
        assert expected == actual


if __name__ == "__main__":
    unittest.main()



import logging
import re
from abc import ABC, abstractmethod
from typing import Optional

from llama_index.core.llms import LLM, MockLLM
from llama_index.llms.openai.utils import O1_MODELS

from unstract.sdk.adapters.base import Adapter
from unstract.sdk.adapters.enums import AdapterTypes
from unstract.sdk.adapters.exceptions import LLMError

logger = logging.getLogger(__name__)


class LLMAdapter(Adapter, ABC):
    def __init__(self, name: str):
        super().__init__(name)
        self.name = name

    @staticmethod
    def get_id() -> str:
        return ""

    @staticmethod
    def get_name() -> str:
        return ""

    @staticmethod
    def get_description() -> str:
        return ""

    @staticmethod
    @abstractmethod
    def get_provider() -> str:
        pass

    @staticmethod
    def get_icon() -> str:
        return ""

    @staticmethod
    def get_adapter_type() -> AdapterTypes:
        return AdapterTypes.LLM

    @staticmethod
    def parse_llm_err(e: Exception) -> LLMError:
        """Parse the error from an LLM provider.

        Helps parse errors from a provider and wraps with custom exception.

        Args:
            e (Exception): Exception from LLM provider

        Returns:
            LLMError: Error to be sent to the user
        """
        return LLMError(str(e), actual_err=e)

    def get_llm_instance(self) -> LLM:
        """Instantiate the llama index LLM class.

        Returns:
            LLM: llama index implementation of the LLM
            Raises exceptions for any error
        """
        return MockLLM()

    @staticmethod
    def _test_llm_instance(llm: Optional[LLM]) -> bool:
        if llm is None:
            raise LLMError(
                message="Unable to connect to LLM, please recheck the configuration",
                status_code=400,
            )
        # Get completion kwargs based on model capabilities
        completion_kwargs = {}
        if hasattr(llm, 'model') and getattr(llm, 'model') not in O1_MODELS:
            completion_kwargs['temperature'] = 0.003
            
        response = llm.complete(
            "The capital of Tamilnadu is ",
            **completion_kwargs
        )
        response_lower_case: str = response.text.lower()
        find_match = re.search("chennai", response_lower_case)
        if find_match:
            return True
        else:
            msg = (
                "LLM based test failed. The credentials was valid however a sane "
                "response was not obtained from the LLM provider, please recheck "
                "the configuration."
            )
            raise LLMError(message=msg, status_code=400)

    def test_connection(self) -> bool:
        try:
            test_result: bool = self._test_llm_instance(llm=self.get_llm_instance())
        except Exception as e:
            # Avoids circular import errors
            from unstract.sdk.adapters.llm.exceptions import parse_llm_err

            raise parse_llm_err(e, llm_adapter=self) from e
        return test_result

    def get_context_window_size(self) -> int:
        """Get the context window size supported by the LLM.

        Note: None of the derived classes implement this method

        Returns:
            int: Context window size supported by the LLM
        """
        context_window_size: int = 0
        llm = self.get_llm_instance()
        if llm:
            context_window_size = llm.metadata.context_window
        return context_window_size



import os
from typing import Any

from llama_index.core.embeddings import BaseEmbedding
from llama_index.embeddings.bedrock import BedrockEmbedding

from unstract.sdk.adapters.embedding.embedding_adapter import EmbeddingAdapter
from unstract.sdk.adapters.embedding.helper import EmbeddingHelper
from unstract.sdk.adapters.exceptions import AdapterError

class Constants:
    MODEL = "model"
    TIMEOUT = "timeout"
    MAX_RETRIES = "max_retries"
    SECRET_ACCESS_KEY = "aws_secret_access_key"
    ACCESS_KEY_ID = "aws_access_key_id"
    REGION_NAME = "region_name"
    DEFAULT_TIMEOUT = 240
    DEFAULT_MAX_RETRIES = 3

class Bedrock(EmbeddingAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("Bedrock")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "bedrock|88199741-8d7e-4e8c-9d92-d76b0dc20c91"

    @staticmethod
    def get_name() -> str:
        return "Bedrock"

    @staticmethod
    def get_description() -> str:
        return "Bedrock Embedding"

    @staticmethod
    def get_provider() -> str:
        return "bedrock"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/Bedrock.png"

    def get_embedding_instance(self) -> BaseEmbedding:
        try:
            embedding_batch_size = EmbeddingHelper.get_embedding_batch_size(
                config=self.config
            )
            embedding: BaseEmbedding = BedrockEmbedding(
                model_name=self.config.get(Constants.MODEL),
                aws_access_key_id=self.config.get(Constants.ACCESS_KEY_ID),
                aws_secret_access_key=self.config.get(Constants.SECRET_ACCESS_KEY),
                region_name=self.config.get(Constants.REGION_NAME),
                timeout=float(
                    self.config.get(Constants.TIMEOUT, Constants.DEFAULT_TIMEOUT)
                ),
                max_retries=int(
                    self.config.get(Constants.MAX_RETRIES, Constants.DEFAULT_MAX_RETRIES)
                ),
                embed_batch_size=embedding_batch_size,
            )
            return embedding
        except Exception as e:
            raise AdapterError(str(e))
        
     


import logging
from typing import Any, Optional

from llama_index.core.embeddings import BaseEmbedding

from unstract.sdk.adapters.exceptions import AdapterError

logger = logging.getLogger(__name__)


class EmbeddingConstants:
    DEFAULT_EMBED_BATCH_SIZE = 10
    EMBED_BATCH_SIZE = "embed_batch_size"


class EmbeddingHelper:
    @staticmethod
    def get_embedding_batch_size(config: dict[str, Any]) -> int:
        if config.get(EmbeddingConstants.EMBED_BATCH_SIZE) is None:
            embedding_batch_size = EmbeddingConstants.DEFAULT_EMBED_BATCH_SIZE
        else:
            embedding_batch_size = int(
                config.get(
                    EmbeddingConstants.EMBED_BATCH_SIZE,
                    EmbeddingConstants.DEFAULT_EMBED_BATCH_SIZE,
                )
            )
        return embedding_batch_size

    @staticmethod
    def test_embedding_instance(embedding: Optional[BaseEmbedding]) -> bool:
        try:
            if embedding is None:
                return False
            response = embedding._get_text_embedding("This is a test")
            if len(response) != 0:
                return True
            else:
                return False
        except Exception as e:
            logger.error(f"Error occured while testing adapter {e}")
            raise AdapterError(str(e))
